#ifndef SCANNERSEWPO_H
#define SCANNERSEWPO_H

#include <cmath>
#include "ScannerSModelclasses.h"
#include "ScannerSAux.h"

static const double Mz=91.1875;
static const double Mw=80.385;
static const double Mz2=Mz*Mz;
static const double Mw2=Mw*Mw;
static const double cw2=Mw2/Mz2;
static const double sw2=1-cw2;
static const double Pi=acos(-1);
static const double alpha_EM=0.007297353;

//Tables STU N-singlets
const double InvCovSTU[3][3]={{473.364,-483.646, -142.783},{-483.646, 636.647,247.912},{-142.783, 247.912, 185.564}};
const double DeltaSTU[3]={-0.0260322,0.0348874,0.0578267};

const double InvCovSTUgfitter[3][3]={{837.303,-898.497,-413.648},{-898.497,1159.68,632.276},{-413.648,632.276,485.891}};
const double STUgfitter[3]={0.03,0.05,0.03};

const double mHref=126;

// Chi2 testers
bool TestSTU_2sigma_2HDM(double mhlight, double mHheavy, double mA, double mHc, double alpha,double beta);
bool TestSTU_2HDM(double mhlight, double mHheavy, double mA, double mHc, double alpha,double beta,double Chi2critical);
bool TestSTU_2sigma_xSM2_mixall(MassRef & Mass, MmixingRef & Mixing);
bool TestSTU_xSM2_mixall(MassRef & Mass, MmixingRef & Mixing,double Chi2critical);
bool TestSTU_2sigma_xSM2_DM(MassRef & Mass, MmixingRef & Mixing);
bool TestSTU_xSM2_DM(MassRef & Mass, MmixingRef & Mixing,double Chi2critical);

// SM functions
double DSsm(double Mass);
double DTsm(double Mass);
double DUsm(double Mass);
double DVsm(double Mass);
double DWsm(double Mass);
double DXsm(double Mass);

double Psi(double A);
double PsiIntegrand(double x,void * params);
double Chi(double A);
double ChiIntegrand(double x,void * params);

// N-singlets functions
double DS_nsinglets(MassRef & Mass, MmixingRef & Mixing,size_t ih);
double DT_nsinglets(MassRef & Mass, MmixingRef & Mixing,size_t ih);
double DU_nsinglets(MassRef & Mass, MmixingRef & Mixing,size_t ih);
double DV_nsinglets(MassRef & Mass, MmixingRef & Mixing,size_t ih);
double DX_nsinglets(MassRef & Mass, MmixingRef & Mixing,size_t ih);
double DW_nsinglets(MassRef & Mass, MmixingRef & Mixing,size_t ih);

//2HDM functions
double F0(double A, double B);
double F1(double A, double B);
double IntegrandF1(double x,void * params);
double G1(double A, double B);
double IntegrandG1(double x,void * params);
double G2func(double A);
double IntegrandG2func(double x,void * params);

double DS_2HDM(double mlight, double mHheavy, double mA, double mcharged,double alpha, double beta);
double DT_2HDM(double mlight, double mHheavy, double mA, double mcharged,double alpha, double beta);
double DU_2HDM(double mlight, double mHheavy, double mA, double mcharged,double alpha, double beta);
double DV_2HDM(double mlight, double mHheavy, double mA, double mcharged,double alpha, double beta);
double DW_2HDM(double mlight, double mHheavy, double mA, double mcharged,double alpha, double beta);
double DX_2HDM(double mlight, double mHheavy, double mA, double mcharged,double alpha, double beta);
bool CheckVirtual(double m1, double m2);


#endif
