#include "ScannerSInterface.h"

using namespace std;

void chose_model(std::ifstream & instream,Potential * & V,Potential * & Vaux){
  int model=checkinputheader(instream); 
  switch(model){
  case(0):V=new Potential;
	  Vaux= new Potential;
	  break;
  case(1):V=new TwoHDM;
  	  Vaux= new TwoHDM;
	  break;
  case(2):V=new xSM2;
  	  Vaux= new xSM2;
	  break;
  Default:cerr<<"ScannerS error: the input file's 1st parameter indicated an unknown model."<<endl;
		cerr<<"You can either chose one of the avalable models or generate a file with ScannerSInput.nb."<<endl;
		cerr<<"Check the manual for more details."<<endl;
		exit(EXIT_FAILURE);
  }
  std::clog<<"Potential model: "<<(*V).name()<<endl;   
}

string ReturnOption(int argc, char ** & argv, std::string option){
//looks for "option" in the terminal arguments, if it finds it, returns the next argument 
  unsigned int pos=0;
  string output;

  while(pos!= argc && string(argv[pos])!=option)
    ++pos;
  if((++pos)<argc)
    output=string(argv[pos]);

  return output;
}

int RedirectOstream(std::ostream & os,std::ofstream & ofsredir,std::streambuf * obuff,int argc, char ** & argv,std::string & filename, std::string option){

  filename=ReturnOption(argc,argv,option.c_str());

  if(filename.size()!=0){
    ofsredir.open(filename.c_str());
    obuff = ofsredir.rdbuf();
    os.rdbuf(obuff);
    return 2;
  }
  else{
    ofsredir.open("model.out");
    obuff = ofsredir.rdbuf();
    os.rdbuf(obuff);
    return 0;
  }
}

int RedirectInfostream(std::ostream & os,std::ofstream & ofsredir,std::streambuf * obuff,int argc, char ** & argv,std::string & filename, std::string option){

  filename=ReturnOption(argc,argv,option.c_str());

  if(filename.size()!=0){
    ofsredir.open(filename.c_str());
    obuff = ofsredir.rdbuf();
    os.rdbuf(obuff);
    return 2;
  }
  else
    return 0;
}

void OpenModelFile(std::ifstream & modelfile, std::string & filepath){
  if (fileExists(filepath.c_str()))
    modelfile.open(filepath.c_str());
  else{
    cerr<<endl<<"ScannerS ERROR: Unable to find the model file."<<endl;
    cerr<<"The model file is missing from ~"<<filepath<<endl;
    exit(EXIT_FAILURE);
  }
}

int OpenInputFile(ifstream & instream,int argc, char ** & argv,std::string & inputfile, std::string option){

  inputfile=ReturnOption(argc,argv,option.c_str());

  if(inputfile.size()!=0){
    instream.open(inputfile.c_str());
    return 2;
  }
  else{
    if(fileExists("model.in")){
      instream.open("model.in");
      inputfile="model.in";
      return 0;
    }
    else{
      cerr << endl<<"ScannerS ERROR: No input filename provided! Please generate model.in, or rerun as:"<<endl<<endl<<"  $ ./ScannerS -i input_filename (...) "<<endl<< endl<<"if your input file has a different name."<< endl<<endl;
      exit(EXIT_FAILURE); //Exit in error
    }
  }
}

void CheckHelpNeeded(int argc, char ** & argv){

  unsigned int pos=0;

  while(pos!= argc && string(argv[pos])!=string("--help"))
    ++pos;
  if((++pos)<=argc){
    cout << endl<<"**** HELP *****************************************" << endl<< endl;
    cout << "To compile and run the program do:"<< endl<< "\t $ make" <<endl<<"\t $ ./ScannerS -i input_file_name"<< endl<<endl<< "To clean the compilation do"<< endl<<"\t $ make clean"<< endl<< endl;
    cout << "To run the program in verbose mode, uncomment the following line in the makefile:" << endl<<"  #MODE=-DVERBOSE"<< endl<< endl<<"by removing the hash, i.e."<<endl<< "  MODE=-DVERBOSE"<< endl << endl;
    cout << "Other runtime options are :"<< endl<< endl;
    cout << "-i input_file_name \t"<<endl<<" **************************************"<<endl<<" Specify the Mathematica produced input file name where to read the potential from."<< endl<< endl;
    cout << "-o output_file_name [this is optional] \t "<<endl<<" **************************************"<<endl<<" Redirect cout to write the results of the scan to the the output file specified. Otherwise the file model.out is created."<< endl<< endl;
    cout << "--nscan  number_of_points_to_generate [this is optional] \t "<<endl<<" **************************************"<<endl<<" Specify the number of points to be generated. If not provided, 1 point will be generated."<< endl<< endl;
    cout << "--seed  seed_value [this is optional] \t "<<endl<<" **************************************"<<endl<<" Specify the seed for the pseudo-random number generator. If not provided, 0 will be used."<< endl<< endl;
    cout << "--model  potential_type [this is optional] \t "<<endl<<" **************************************"<<endl<<"Generate a model input file for the potential type chosen (current choices are 2HDM and xSM2)."<< endl<< endl;
    cout << "--log log_file_name [this is optional] \t "<<endl<<" **************************************"<<endl<<" Redirect std::clog to write the information printed on screen for the user,  to the the log file specified."<< endl<< endl;
    cout << "--err err_file_name [this is optional] \t "<<endl<<" **************************************"<<endl<<" Redirect cerr to write the errors printed on screen for the user, to the the error file specified."<< endl<< endl;
    cout << "--model model_name [this is optional] \t "<<endl<<" **************************************"<<endl<<" Generates an input file to the specified type (currently xSM2 and 2HDM)."<< endl<< endl;
    cout<< "NOTE: The default output file is created with the name model.out"<< endl<< endl; 
    cout<< "Compile in verbose mode: If you want to recompile the code in verbose mode with very detailed info of each point that was attempted, follow these steps:"<< endl;
    cout <<" \t $make clean"<< endl;
    cout <<" \t $make MODE=-DVERBOSE"<< endl<< endl; 

    cout << "**** END OF HELP *****************************************"<< endl<< endl;
    exit(EXIT_SUCCESS);
  }
}

void getvariable(std::istream & in,int & i,std::string keyword){
//looks for the keyword in the file and gets the 1 values writen after it
  string reader;
  bool check=false;
  while(!in.eof() && !check){
    in>>reader;
    if (reader==keyword){
      in>>i;
      check=true;
    }
  }
  if(!check){
    cerr<<endl<<"ScannerS ERROR: Failed to find "<<keyword<<" on the input file."<<endl;
    exit(EXIT_FAILURE);
  }
  else
    in.seekg(0,in.beg);
}

bool getvariable(std::istream & in,double & min,double & max,std::string keyword){
//looks for the keyword in the file and gets the 2 values writen after it
  string reader;
  bool check=false;
  while(!in.eof() && !check){
    in>>reader;
    if (reader==keyword){
      in>>min>>max;
      check=true;
    }
  }
  in.clear();	//clears the eof flag
  in.seekg(0,in.beg);
  if(check)
    return true;
  else
    return false;
}

int checkinputheader(ifstream & instream){
  string str;
  do{
	instream>>str;
	if(!instream){
	  cerr<<"ScannerS error (in checkinputheader()): header missing in input file."<<endl;
	  exit(EXIT_FAILURE);
	}
  }while(str!=">Potential_type:");
  int model;
  instream>>model;
  return model;
}

void MakeInputfile(int argc, char ** & argv,std::string keyword){
//Creates a model input file for chosen model if the --model was called in the terminal
  string modeltype=ReturnOption(argc,argv,keyword);
  if(!modeltype.empty()){
    if(modeltype=="2HDM"){
      std::clog<<endl<<"Generating a 2HDM model file (TwoHDM.in) in this folder."<<endl<<endl;  
      ofstream modelfile;
      modelfile.open("TwoHDM.in");
      modelfile<<"#####--------------------------------------------------------------#####"<<endl;
      modelfile<<"#####----------------------------ScannerS--------------------------#####"<<endl;
      modelfile<<endl;
      modelfile<<"This is an input file for ScannerS."<<endl;
      modelfile<<"When running ScannerS with this file the program will be loaded with the 2HDM potential."<<endl;
      modelfile<<"Fill the variables bellow in order to specify the ranges you wish to confine the potential parameters."<<endl;
      modelfile<<"These are the following:"<<endl;
      modelfile<<"- Light Higgs mass		mh"<<endl;
      modelfile<<"- Heavy Higgs mass		mH"<<endl;
      modelfile<<"- Other scalar mass		mA"<<endl;
      modelfile<<"- Charged Higgs mass		mH+"<<endl;
      modelfile<<"- Mixing angle			alpha"<<endl;
      modelfile<<"- v1/v2 angle (tan)		tanbeta"<<endl;
      modelfile<<"- m12^2 coupling		m12^2 "<<endl;
      modelfile<<endl;
      modelfile<<"Besides the independent parameters above you can also chose to restrict the other the values for the other couplings. You can do this by adding their min and max values below. These are m11^2, m22^2, lambda1, lambda2, lambda3, lambda4 and lambda5 (don't forget to add the '>' when declaring them)."<<endl;
      modelfile<<"If you declare any of these parameters range it will be checked by CheckBounds() and points outside this range will  be rejected."<<endl;
      modelfile<<endl;
      modelfile<<"DO NOT ALTER THE TABLE FORMAT/ORDER."<<endl;
      modelfile<<endl;
      modelfile<<"DO NOT REMOVE THIS HEADER."<<endl;
      modelfile<<endl;
      modelfile<<"#####--------------------------------------------------------------#####"<<endl;
      modelfile<<endl;
      modelfile<<"### START OF INPUT ###"<<endl;
      modelfile<<">Potential_type: 1 (0: Mathematica generated; 1: 2HDM; 2: xSM2)"<<endl;
      modelfile<<endl;
      modelfile<<">2HDM_type: 1 (1: Type I; 2: Type II; 3: Lepton Specific; 4: Flipped)"<<endl;
      modelfile<<endl;
      modelfile<<"Parameter	MIN	MAX"<<endl;
      modelfile<<">mH		125	500"<<endl;
      modelfile<<">mh		125	125"<<endl;
      modelfile<<">mA		50	500"<<endl;
      modelfile<<">mH+		50	500"<<endl;
      modelfile<<">alpha		-1.5707	1.5707"<<endl;
      modelfile<<">tanbeta	2.5	50"<<endl;
      modelfile<<">m12^2		-2500	90000"<<endl;
      modelfile<<endl;
      modelfile<<"### END OF INPUT ###"<<endl;
      
      modelfile.close();
      exit(EXIT_SUCCESS);
    }
    else if(modeltype=="xSM2"){
      std::clog<<endl<<"Generating a xSM2 model file (xSM2.in) in this folder."<<endl<<endl;  
      ofstream modelfile;
      modelfile.open("xSM2.in");
      modelfile<<"#####--------------------------------------------------------------#####"<<endl;
      modelfile<<"#####----------------------------ScannerS--------------------------#####"<<endl;
      modelfile<<endl;
      modelfile<<"This is an input file for ScannerS."<<endl;
      modelfile<<"When running ScannerS with this file the program will be loaded with the xSM2 potential."<<endl;
      modelfile<<"This model has two main possible phases. Either the imaginary component of S (va) has a 0 VeV and the model has a dark matter candidate, or there are 3 non-zero VeV's that mix (see arxiv:1301.2599 for more details). If va=0 all three non-goldstone masses and the one of the coupling (by default a1) are independent and will be randomly generated. If vs!=0 the only independent parameter is Higgs' mass."<<endl;
      modelfile<<"Fill the variables bellow in order to specify the ranges you wish to confine the potential parameters."<<endl;
      modelfile<<"These are the following:"<<endl;
      modelfile<<"-Higgs VeV			vh"<<endl;
      modelfile<<"-Re[S] VeV			vs"<<endl;
      modelfile<<"-Im[S] VeV			va"<<endl;
      modelfile<<"-Higgs mass			mh"<<endl;
      modelfile<<"-First new scalar mass		ms1"<<endl;
      modelfile<<"-Second new scalar mass		ms2"<<endl;
      modelfile<<"-Cubic coupling to S		a1"<<endl;
      modelfile<<endl;
      modelfile<<"Besides the independent parameters above you can also chose to restrict the other the values for the other couplings. You can do this by adding their min and max values below. These are msq, lambda, delta2, b2, d2 and b1 (don't forget to add the '>' when declaring them)."<<endl;
      modelfile<<"If you declare any of these parameters range it will be checked by CheckBounds() and points outside this range will  be rejected."<<endl;
      modelfile<<endl;
      modelfile<<"Note: Since in the mix_all phase ms1, ms2 and a1 are not independent, if you do not wish to restrict them you can comment or delete their range entry. In the dark matter case, if you wish to have different ranges for the new particles, note that the decoupled one is ms2."<<endl;
      modelfile<<endl;
      modelfile<<"DO NOT ALTER THE TABLE FORMAT/ORDER."<<endl;
      modelfile<<endl;
      modelfile<<"DO NOT REMOVE THIS HEADER."<<endl;
      modelfile<<endl;
      modelfile<<"#####--------------------------------------------------------------#####"<<endl;
      modelfile<<endl;
      modelfile<<"### START OF INPUT ###"<<endl;
      modelfile<<">Potential_type: 2 (0: Mathematica generated; 1: 2HDM; 2: xSM2)"<<endl;
      modelfile<<endl;
      modelfile<<"Parameter	MIN		MAX"<<endl;
      modelfile<<">vh		246		246"<<endl;
      modelfile<<">vs		0		500"<<endl;
      modelfile<<">va		0		500"<<endl;
      modelfile<<">mh		125.9		125.9"<<endl;
      modelfile<<">ms1		10		500"<<endl;
      modelfile<<">ms2		10		500"<<endl;
      modelfile<<">a1		-100000		100000"<<endl;
      modelfile<<endl;
      modelfile<<"### END OF INPUT ###"<<endl;
      
      modelfile.close();
      exit(EXIT_SUCCESS);
    }
    else{
      cerr<<endl<<"ScannerS ERROR: The model type you tired to make an input file for with --model does not exist."<<endl;
      cerr<<"The arguments for these command are 2HDM or xSM2."<<endl<<endl;
      exit(EXIT_FAILURE);
    }
  }  
}
