(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     89961,       2135]
NotebookOptionsPosition[     86791,       2031]
NotebookOutlinePosition[     87227,       2048]
CellTagsIndexPosition[     87184,       2045]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["ScannerS Input file generator", "Title",
 CellChangeTimes->{{3.534139124702258*^9, 3.534139140894373*^9}, {
  3.5431344646773643`*^9, 3.543134466549079*^9}}],

Cell[TextData[{
 "This notebook allows the user to generate an input file for the ScannerS \
program, given a polynomial expression for the scalar potential and the \
number of real components that compose the various scalar multiplets. The \
first section needs to be edited for each particular model and the key \
information to provide is: i)The number of real components defining the \
variable Nreal and the number of couplings Ncoup ii) A fully expanded \
expression for the scalar potential decomposed in Nreal scalars organised in \
an array \[Phi][i] with i=1...Nreal, and in terms of the couplings L[j], \
j=1...Ncoup and respective ranges for the scan iii) The path and name where \
to save the ScannerS input file\n\nSome other switches are also provided to \
control (for example) user defined parametrizations.\n\n",
 StyleBox["The list of quantities that must be defined is",
  FontWeight->"Bold",
  FontColor->RGBColor[0, 0, 1]],
 " (descriptions below in comments of each variable):\n-> ",
 StyleBox["Nreal ",
  FontWeight->"Bold"],
 "\n-> ",
 StyleBox["Ncoup ",
  FontWeight->"Bold"],
 "\n-> ",
 StyleBox["NparamsVEVs\n-> NparamsMix",
  FontWeight->"Bold"],
 "\n->",
 StyleBox[" NparamsML",
  FontWeight->"Bold"],
 "\n->",
 StyleBox[" Vscalar",
  FontWeight->"Bold"],
 "\n-> ",
 StyleBox["\[Phi]Min[i] ",
  FontWeight->"Bold"],
 " and ",
 StyleBox["\[Phi]Max[i]",
  FontWeight->"Bold"],
 " (i=0, ... ,Nreal-1 )\n-> ",
 StyleBox["\[Phi]ParMin[j]",
  FontWeight->"Bold"],
 "  and ",
 StyleBox["\[Phi]ParMax[j]",
  FontWeight->"Bold"],
 " (j=0, ... , NparamsVEVs-1 ) --- OPTIONAL (only if  NparamsVEVs>0)\n-> ",
 StyleBox["MixParMin[j] ",
  FontWeight->"Bold"],
 " and ",
 StyleBox["MixParMax[j]",
  FontWeight->"Bold"],
 " (j=0, ... , NparamsMix-1 ) --- OPTIONAL (only if  NparamsMix>0)\n-> L",
 StyleBox["Min[k]",
  FontWeight->"Bold"],
 " and L",
 StyleBox["Max[k]",
  FontWeight->"Bold"],
 " (k=0, ... , Ncoup-1)\n-> ",
 StyleBox["massMin[i]",
  FontWeight->"Bold"],
 "  and ",
 StyleBox["massMax[i]",
  FontWeight->"Bold"],
 " (i=0, ... ,Nreal-1 )\n-> ",
 StyleBox["MLParMin[n]",
  FontWeight->"Bold"],
 " and ",
 StyleBox["MLParMax[n]",
  FontWeight->"Bold"],
 " (n=0, ... , NparamsML-1) --- OPTIONAL (only if  NparamsML>0)\n-> ",
 StyleBox["InputFileName",
  FontWeight->"Bold"]
}], "Text",
 CellChangeTimes->{{3.534139153767818*^9, 3.5341394862417727`*^9}, {
   3.534140210077389*^9, 3.534140226239519*^9}, {3.534140287550242*^9, 
   3.534140364231353*^9}, {3.534140503439558*^9, 3.534140510794918*^9}, {
   3.5341447449859133`*^9, 3.534144752493857*^9}, {3.585478586019114*^9, 
   3.585478638814126*^9}, {3.585479527066248*^9, 3.585479605113472*^9}, {
   3.585482684075522*^9, 3.5854827830035458`*^9}, {3.585483502866034*^9, 
   3.585483773446186*^9}, {3.5854838073725777`*^9, 3.585483846521552*^9}, {
   3.5854839707672043`*^9, 3.58548400564085*^9}, {3.585484332685555*^9, 
   3.585484357047865*^9}, {3.585499880744681*^9, 3.585499906893263*^9}, {
   3.585500280273981*^9, 3.5855002806999063`*^9}, {3.585500334549485*^9, 
   3.5855003604343967`*^9}, 3.585500400530427*^9, {3.585500459233739*^9, 
   3.58550049324888*^9}, {3.585500533622078*^9, 3.585500543706848*^9}, {
   3.601101505653954*^9, 3.601101505804448*^9}, {3.601128563004146*^9, 
   3.6011285632507763`*^9}, {3.601128881197302*^9, 3.601128881392869*^9}, {
   3.601189726643999*^9, 3.601189743147997*^9}}],

Cell[CellGroupData[{

Cell["User defined expressions (To be edited by the user)", "Section",
 CellChangeTimes->{{3.534139699550837*^9, 3.534139703527507*^9}, {
  3.534152333239427*^9, 3.534152342256386*^9}}],

Cell[CellGroupData[{

Cell["Internal switches to control model and run", "Subsection",
 CellChangeTimes->{{3.5341400938604507`*^9, 3.534140099025897*^9}, {
  3.534140230565733*^9, 3.5341402323063917`*^9}, {3.585478545532669*^9, 
  3.5854785537190104`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Nreal", "=", "8"}], ";"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"This", " ", "is", " ", "the", " ", "number", " ", "of"}], ",", 
    " ", "real", ",", " ", 
    RowBox[{"degrees", " ", "of", " ", 
     RowBox[{"freedom", "/", "fields"}]}]}], " ", "*)"}]}]], "Input",
 CellChangeTimes->{{3.534140001122467*^9, 3.53414002733514*^9}, {
   3.535107537502141*^9, 3.535107539298517*^9}, {3.549553504718693*^9, 
   3.54955350476075*^9}, {3.5496185414294558`*^9, 3.5496185415172157`*^9}, {
   3.583119945326511*^9, 3.583119960657104*^9}, 3.583126321533564*^9, {
   3.585478556417615*^9, 3.585478580868178*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Ncoup", "=", "8"}], ";"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"This", " ", "is", " ", "the", " ", "number", " ", "of"}], ",", 
    " ", "real", ",", " ", 
    RowBox[{"couplings", " ", "in", " ", "the", " ", "potential"}]}], " ", 
   "*)"}]}]], "Input",
 CellChangeTimes->{{3.534140236071824*^9, 3.534140262292913*^9}, {
   3.535107550204233*^9, 3.535107551069602*^9}, {3.535115146643319*^9, 
   3.535115148652267*^9}, {3.537020277634386*^9, 3.537020277848011*^9}, {
   3.549618546323228*^9, 3.549618571938922*^9}, {3.549621056195997*^9, 
   3.549621056242008*^9}, {3.549621150023106*^9, 3.549621173023985*^9}, {
   3.549638953324149*^9, 3.549638953615148*^9}, {3.5497100916771917`*^9, 
   3.549710092642866*^9}, {3.583071724970649*^9, 3.583071726395556*^9}, 
   3.583126323095623*^9, {3.585478641804598*^9, 3.585478662795334*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"NparamsVEVs", "=", "1"}], ";"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
     "This", " ", "is", " ", "active", "  ", "only", " ", "if", " ", "larger",
       " ", "than", " ", "zero", " ", 
      RowBox[{
       RowBox[{"(", "0", ")"}], ".", " ", "To"}], " ", "de"}], "-", 
     RowBox[{"activate", " ", "set", " ", "to", " ", 
      RowBox[{"zero", ".", " ", "This"}], " ", "integer", " ", "is", " ", 
      "the", " ", "number", " ", "of", " ", "real", " ", "parameters", " ", 
      "used", " ", "to", " ", "re"}], "-", 
     RowBox[{"parametrize", " ", "the", " ", "VEVs", " ", "being", " ", 
      RowBox[{"generated", ".", " ", "If"}], " ", "off", " ", 
      RowBox[{"(", "0", ")"}]}]}], ",", " ", 
    RowBox[{
    "the", " ", "VEVs", " ", "are", " ", "generated", " ", "uniformly", " ", 
     "in", " ", "a", " ", "hypercubic", " ", "box", " ", 
     RowBox[{"(", 
      RowBox[{
      "according", " ", "to", " ", "the", " ", "ranges", " ", "defined", " ", 
       "in", " ", "the", " ", "section", " ", "\"\<Ranges\>\"", " ", 
       "below"}], ")"}]}], ",", " ", 
    RowBox[{
     RowBox[{"otherwise", " ", "the", " ", "user"}], "-", 
     RowBox[{
     "defined", " ", "function", " ", "\"\<MyPhiParametrization\>\"", " ", 
      "in", " ", 
      RowBox[{"ScannerSUser", ".", "cpp"}], " ", "is", " ", "used"}]}]}], 
   "*)"}]}]], "Input",
 CellChangeTimes->{{3.5830717287197104`*^9, 3.5830717580937147`*^9}, {
   3.583071840934731*^9, 3.5830718548112097`*^9}, 3.583119290786797*^9, {
   3.583119758713336*^9, 3.5831197587669573`*^9}, 3.5831253374820137`*^9, 
   3.583126325985382*^9, {3.5831403702663116`*^9, 3.583140371724491*^9}, {
   3.583140438864959*^9, 3.5831404430164022`*^9}, {3.583140514757739*^9, 
   3.5831405154071712`*^9}, {3.5831415397580347`*^9, 3.583141540463245*^9}, {
   3.583141895670452*^9, 3.5831419017159443`*^9}, {3.583758406962304*^9, 
   3.5837584344579268`*^9}, 3.5837585071671047`*^9, {3.5854786707028017`*^9, 
   3.585478831309935*^9}, 3.585478893914448*^9, {3.585479014687324*^9, 
   3.585479043291883*^9}, {3.585479136162771*^9, 3.585479145000149*^9}, {
   3.585500078912011*^9, 3.585500082639312*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"NparamsMix", "=", "0"}], ";"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
     "This", " ", "is", " ", "active", "  ", "only", " ", "if", " ", "larger",
       " ", "than", " ", "zero", " ", 
      RowBox[{
       RowBox[{"(", "0", ")"}], ".", " ", "To"}], " ", "de"}], "-", 
     RowBox[{"activate", " ", "set", " ", "to", " ", 
      RowBox[{"zero", ".", " ", "This"}], " ", "integer", " ", "is", " ", 
      "the", " ", "number", " ", "of", " ", "real", " ", "parameters", " ", 
      "used", " ", "to", " ", "parametrize", " ", "the", " ", "mixing", " ", 
      "matrix", " ", "through", " ", "a", " ", "defined", " ", "function", 
      " ", "\"\<MyCoupMassRelations\>\"", " ", "in", " ", 
      RowBox[{"ScannerSUser", ".", "cpp", ".", " ", "If"}], " ", "off", " ", 
      RowBox[{"(", "0", ")"}]}]}], ",", 
    RowBox[{
    "the", " ", "mixing", " ", "matrix", " ", "is", " ", "generated", " ", 
     "with", " ", "uniformly", " ", "probability", " ", 
     RowBox[{"(", 
      RowBox[{
      "with", " ", "respect", " ", "to", " ", "the", " ", "H\[ADoubleDot]ar", 
       " ", "measure"}], ")"}]}], ",", " ", 
    RowBox[{"otherwise", " ", "the"}], ",", " ", 
    RowBox[{"user", "-", "defined"}], ",", " ", 
    RowBox[{"function", " ", "\"\<MyCoupMassRelations\>\"", " ", "in", " ", 
     RowBox[{"ScannerSUser", ".", "cpp"}], " ", "is", " ", 
     RowBox[{"used", ".", " ", "Note"}], " ", "that", " ", "such", " ", 
     "user", " ", "defined", " ", "function", " ", "can", " ", "depend", " ", 
     "of", " ", "the", " ", "VEV", " ", "values", " ", "or", " ", 
     "parameters"}]}], "*)"}]}]], "Input",
 CellChangeTimes->{{3.585479369598381*^9, 3.585479467597618*^9}, {
   3.585485066284457*^9, 3.585485122966332*^9}, {3.585485199168365*^9, 
   3.585485199442231*^9}, 3.5854854228579197`*^9, {3.585495474345923*^9, 
   3.5854954760813427`*^9}, {3.585495509304283*^9, 3.585495509595584*^9}, {
   3.585495629862289*^9, 3.585495630128745*^9}, {3.585499914078368*^9, 
   3.585500046258539*^9}, {3.585500095253937*^9, 3.5855001662337313`*^9}, {
   3.585501078206011*^9, 3.585501078303528*^9}, {3.5855023471534023`*^9, 
   3.585502347376927*^9}, 3.6011285633701487`*^9, 3.603543038029784*^9, {
   3.6064778776916103`*^9, 3.606477878118513*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"NparamsML", " ", "=", " ", "0"}], ";"}], 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{
     "This", " ", "is", " ", "active", "  ", "only", " ", "if", " ", "larger",
       " ", "than", " ", "zero", " ", 
      RowBox[{
       RowBox[{"(", "0", ")"}], ".", " ", "To"}], " ", "de"}], "-", 
     RowBox[{"activate", " ", "set", " ", "to", " ", "zero", " ", 
      RowBox[{
       RowBox[{"(", "0", ")"}], ".", " ", "This"}], " ", "integer", " ", "is",
       " ", "the", " ", "number", " ", "of", " ", "real", " ", "parameters", 
      " ", "used", " ", "to", " ", "parametrize", " ", "a", " ", "relation", 
      " ", "among", " ", "the", " ", "last", " ", "generated", " ", "Masses", 
      " ", "and", " ", "couplings", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
        "see", " ", "user", " ", "defined", " ", "function", " ", 
         "\"\<MyCoupMassRelations\>\"", " ", "in", " ", 
         RowBox[{"ScannerSUser", ".", "cpp"}]}], ")"}], ".", " ", "If"}], " ",
       "off", " ", 
      RowBox[{"(", "0", ")"}]}]}], ",", " ", 
    RowBox[{
    "the", " ", "last", " ", "independent", " ", "masses", " ", "and", " ", 
     "couplings", " ", "are", " ", "generated", " ", "in", " ", "a", " ", 
     "hypercubic", " ", "box", " ", 
     RowBox[{"(", 
      RowBox[{
      "according", " ", "to", " ", "the", " ", "ranges", " ", "defined", " ", 
       "in", " ", "the", " ", "section", " ", "\"\<Ranges\>\"", " ", 
       "below"}], ")"}]}], ",", " ", 
    RowBox[{"otherwise", " ", "the"}], ",", " ", 
    RowBox[{"user", "-", "defined"}], ",", " ", 
    RowBox[{"function", " ", "\"\<MyCoupMassRelations\>\"", " ", "in", " ", 
     RowBox[{"ScannerSUser", ".", "cpp"}], " ", "is", " ", 
     RowBox[{"used", ".", " ", "Note"}], " ", "that", " ", "this", " ", 
     "user", " ", "defined", " ", "function", " ", "can", " ", "depend", " ", 
     "on", " ", "VEVs", " ", "and", " ", "Mixings", " ", "or", " ", 
     "corresponding", " ", 
     RowBox[{"parameters", "."}]}]}], " ", "*)"}]}]], "Input",
 CellChangeTimes->{{3.5837583148484592`*^9, 3.583758386966074*^9}, {
   3.583758420229499*^9, 3.583758435914453*^9}, {3.5837643032246428`*^9, 
   3.5837643034445667`*^9}, {3.583764495911582*^9, 3.5837644967409887`*^9}, {
   3.5837653238688374`*^9, 3.583765324067956*^9}, 3.585478843101055*^9, 
   3.585478929658777*^9, {3.585478981298455*^9, 3.585479011929343*^9}, {
   3.5854790644483423`*^9, 3.585479153117096*^9}, {3.585500062479171*^9, 
   3.58550007217819*^9}, {3.58550017750421*^9, 3.585500209571487*^9}, 
   3.601128881508999*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Polynomial Expression for the potential in terms of \[Phi][i] and L[j]\
\>", "Subsection",
 CellChangeTimes->{{3.534140137138444*^9, 3.534140159456518*^9}, {
   3.534140270579125*^9, 3.534140272374899*^9}, {3.534140376055615*^9, 
   3.534140382933145*^9}, {3.534140544931155*^9, 3.5341405499086857`*^9}, 
   3.601101505853801*^9}],

Cell["\<\
This first lines are just auxiliary expressions that are optional. What is \
important is that the Vscalar expression is defined correctly some lines \
below!\
\>", "Text",
 CellChangeTimes->{{3.585479652906949*^9, 3.585479713674541*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalPhi]1", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[Phi]", "[", "0", "]"}], "+", 
       RowBox[{"I", " ", 
        RowBox[{"\[Phi]", "[", "1", "]"}]}]}], ")"}], "/", 
     RowBox[{"Sqrt", "[", "2", "]"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[Phi]", "[", "2", "]"}], "+", 
       RowBox[{"I", " ", 
        RowBox[{"\[Phi]", "[", "3", "]"}]}]}], ")"}], "/", 
     RowBox[{"Sqrt", "[", "2", "]"}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.535109196021991*^9, 3.535109196373943*^9}, {
  3.535109520871797*^9, 3.535109528823069*^9}, {3.535110070367919*^9, 
  3.535110071019047*^9}, {3.535111702122002*^9, 3.535111702444166*^9}, {
  3.549553587505783*^9, 3.549553597393755*^9}, {3.5496187395042467`*^9, 
  3.5496187613882113`*^9}, {3.565089691920523*^9, 3.565089706237021*^9}, {
  3.565089750073328*^9, 3.565089751799815*^9}, {3.56509265539555*^9, 
  3.565092659255678*^9}, {3.570780201546114*^9, 3.570780246360174*^9}, {
  3.58312642496203*^9, 3.583126433395574*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"\[Phi]", "[", "0", "]"}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      RowBox[{"\[Phi]", "[", "1", "]"}]}]}], 
    SqrtBox["2"]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"\[Phi]", "[", "2", "]"}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      RowBox[{"\[Phi]", "[", "3", "]"}]}]}], 
    SqrtBox["2"]]}], "}"}]], "Output",
 CellChangeTimes->{
  3.570347593155091*^9, 3.570780234626966*^9, 3.570780275568789*^9, 
   3.570782347770499*^9, 3.570801957205337*^9, 3.57080200088017*^9, 
   3.570802122705267*^9, 3.570802161340332*^9, 3.570802221310916*^9, 
   3.570802303044646*^9, 3.573370250600205*^9, {3.577100674377851*^9, 
   3.577100683118408*^9}, 3.577100911586241*^9, 3.577102192539304*^9, 
   3.5831268402225933`*^9, 3.5831271776408167`*^9, 3.5831403762780447`*^9, 
   3.583140445930541*^9, 3.583140517702251*^9, 3.583141544631421*^9, 
   3.583141904318516*^9, 3.5831436207281103`*^9, 3.583143650747457*^9, 
   3.583758931657578*^9, 3.583764307231513*^9, 3.583764500057564*^9, 
   3.583765327523185*^9, {3.5854800403238163`*^9, 3.585480057166016*^9}, {
   3.5854801491305437`*^9, 3.585480158899188*^9}, {3.585480200166749*^9, 
   3.585480208771296*^9}, {3.585480503981059*^9, 3.5854805308222647`*^9}, {
   3.585480828383356*^9, 3.585480844897043*^9}, 3.585481011899619*^9, {
   3.585481261582512*^9, 3.585481292022044*^9}, 3.5854814295871897`*^9, {
   3.585483861915666*^9, 3.5854838691263123`*^9}, {3.585484991600119*^9, 
   3.5854849947913113`*^9}, {3.585485069364324*^9, 3.585485080941546*^9}, 
   3.585485127057515*^9, 3.5854852045462627`*^9, 3.5854854263313293`*^9, 
   3.58549547930477*^9, 3.585495513160664*^9, 3.585495632889965*^9, {
   3.58550100274291*^9, 3.585501024325069*^9}, 3.585501080793311*^9, 
   3.58550163087674*^9, 3.5855023512767973`*^9, {3.585657173065467*^9, 
   3.5856571921535873`*^9}, 3.5856572470992403`*^9, 3.585657371909479*^9, 
   3.585657569186314*^9, 3.585657746198563*^9, {3.585657826232843*^9, 
   3.585657883929962*^9}, 3.5856579146957817`*^9, 3.587798775334564*^9, {
   3.592286715416472*^9, 3.592286742840147*^9}, 3.5922867809220457`*^9, 
   3.592289779519063*^9, 3.59229029687252*^9, 3.59229049537166*^9, 
   3.592290628947899*^9, 3.59229083293758*^9, 3.592290985571411*^9, {
   3.5922910353437977`*^9, 3.592291061138936*^9}, 3.5922911415331507`*^9, 
   3.592291355716724*^9, {3.5922915007096777`*^9, 3.592291513267003*^9}, {
   3.5922915771211*^9, 3.592291606524705*^9}, {3.592291913358042*^9, 
   3.592291947301407*^9}, 3.592292335284577*^9, 3.592292814131658*^9, 
   3.592301637967435*^9, 3.592302726493981*^9, 3.592303127534182*^9, 
   3.5923065008244553`*^9, 3.592317514985449*^9, 3.592317549452601*^9, 
   3.592317597415447*^9, 3.59231843647124*^9, 3.592318797247834*^9, {
   3.592318884748185*^9, 3.592318913548868*^9}, 3.5923189623752832`*^9, 
   3.592319188745249*^9, 3.592319379813239*^9, 3.595334685486149*^9, 
   3.595334726413558*^9, 3.595422766008196*^9, 3.6001718488529053`*^9, 
   3.600171896570663*^9, 3.6001727717834663`*^9, 3.60017529704252*^9, 
   3.6010996847987127`*^9, {3.601099763955369*^9, 3.60109978777964*^9}, 
   3.601099826527711*^9, 3.6011001030200157`*^9, 3.601100547200789*^9, {
   3.601100651402573*^9, 3.601100690814116*^9}, 3.601100885275361*^9, 
   3.60110104094063*^9, 3.601101526390029*^9, 3.601101574778414*^9, 
   3.6011017388532877`*^9, 3.601128655446451*^9, 3.60112869563371*^9, {
   3.601128796306066*^9, 3.6011288251870203`*^9}, {3.6011289476674013`*^9, 
   3.60112895709371*^9}, {3.601189811657422*^9, 3.60118982097681*^9}, 
   3.601206273738933*^9, 3.6028624013749237`*^9, 3.602865085361226*^9, 
   3.6028651236965218`*^9, 3.602865186411645*^9, 3.6028654905402184`*^9, 
   3.602866886984942*^9, 3.602866918090953*^9, 3.602918622221496*^9, 
   3.602919047716147*^9, {3.602919103693698*^9, 3.602919133166088*^9}, 
   3.6029192481172333`*^9, 3.602919285836516*^9, 3.602920098960024*^9, {
   3.602920163263307*^9, 3.60292018548003*^9}, 3.602920226916732*^9, {
   3.602920269560336*^9, 3.602920296242845*^9}, 3.6029304199806013`*^9, {
   3.602930480929291*^9, 3.6029305236003323`*^9}, 3.602930557200156*^9, {
   3.602930597810452*^9, 3.6029306251033382`*^9}, 3.602930772411339*^9, 
   3.602931108751423*^9, 3.603543064770714*^9, 3.603543667885869*^9, 
   3.603543776380711*^9, 3.6035439474969254`*^9, 3.606477949469735*^9, 
   3.608794461013199*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalPhi]1Dag", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[Phi]", "[", "0", "]"}], "-", 
       RowBox[{"I", " ", 
        RowBox[{"\[Phi]", "[", "1", "]"}]}]}], ")"}], "/", 
     RowBox[{"Sqrt", "[", "2", "]"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[Phi]", "[", "2", "]"}], "-", 
       RowBox[{"I", " ", 
        RowBox[{"\[Phi]", "[", "3", "]"}]}]}], ")"}], "/", 
     RowBox[{"Sqrt", "[", "2", "]"}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.535109199729326*^9, 3.535109200029863*^9}, {
  3.53510953462142*^9, 3.535109535164555*^9}, {3.535110074312305*^9, 
  3.535110079458116*^9}, {3.535111706207088*^9, 3.535111706549427*^9}, {
  3.535115410604756*^9, 3.535115410974568*^9}, {3.549553589272149*^9, 
  3.549553589294136*^9}, {3.5495536303617373`*^9, 3.5495536304272413`*^9}, {
  3.5496187412780724`*^9, 3.549618763241658*^9}, {3.565089712866297*^9, 
  3.565089721854927*^9}, {3.565092663498235*^9, 3.565092664409509*^9}, {
  3.566636917275683*^9, 3.566636918645827*^9}, {3.566637118684303*^9, 
  3.566637127880097*^9}, {3.566637290440206*^9, 3.566637293504091*^9}, {
  3.570780263370391*^9, 3.570780271608928*^9}, {3.583126435541141*^9, 
  3.583126441786715*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"\[Phi]", "[", "0", "]"}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      RowBox[{"\[Phi]", "[", "1", "]"}]}]}], 
    SqrtBox["2"]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"\[Phi]", "[", "2", "]"}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      RowBox[{"\[Phi]", "[", "3", "]"}]}]}], 
    SqrtBox["2"]]}], "}"}]], "Output",
 CellChangeTimes->{
  3.535109536136553*^9, 3.53511297970275*^9, 3.535115165947134*^9, {
   3.535115404347159*^9, 3.535115411975111*^9}, 3.535278059564253*^9, 
   3.535784418607324*^9, 3.535805282405831*^9, 3.535806163597977*^9, {
   3.536384721430085*^9, 3.536384767407791*^9}, 3.537020508222603*^9, 
   3.5370205964999437`*^9, 3.537189598911167*^9, 3.547361880431159*^9, 
   3.5473653958339252`*^9, 3.547365821748208*^9, 3.547905703027606*^9, 
   3.5484829144440928`*^9, {3.54955362680056*^9, 3.5495536318513927`*^9}, 
   3.5495540396642313`*^9, 3.549555530541011*^9, 3.549555848016901*^9, {
   3.549555997320868*^9, 3.5495560236449003`*^9}, 3.5495669493263206`*^9, 
   3.549600643604191*^9, 3.5496006911698027`*^9, {3.5496187228588057`*^9, 
   3.5496187485891*^9}, 3.5496194167279177`*^9, 3.549620983841875*^9, 
   3.5496210632086153`*^9, {3.54962115410704*^9, 3.549621179388376*^9}, 
   3.54962165502177*^9, 3.549621706293095*^9, 3.5497068453436832`*^9, 
   3.549711878065778*^9, 3.552302631944517*^9, 3.5523027325171328`*^9, 
   3.552302812672686*^9, 3.5523055028230762`*^9, 3.552842049456295*^9, 
   3.555738441132964*^9, 3.555741444525577*^9, 3.570347593438706*^9, 
   3.570780278138885*^9, 3.570782347948301*^9, 3.570801957254089*^9, 
   3.570802000921617*^9, 3.570802122868432*^9, 3.570802161418197*^9, 
   3.570802221419132*^9, 3.570802303156346*^9, 3.573370250693198*^9, {
   3.577100674596106*^9, 3.5771006831698847`*^9}, 3.5771009117233*^9, 
   3.577102192738039*^9, 3.583126840330243*^9, 3.5831271778269787`*^9, 
   3.583140376533866*^9, 3.583140446138136*^9, 3.5831405177246304`*^9, 
   3.583141544741438*^9, 3.5831419045702753`*^9, 3.583143620749711*^9, 
   3.583143650770864*^9, 3.583758931885228*^9, 3.583764307427608*^9, 
   3.583764500192922*^9, 3.583765327791109*^9, {3.5854800403485622`*^9, 
   3.585480057382058*^9}, {3.585480149161914*^9, 3.585480158929309*^9}, {
   3.5854802003032093`*^9, 3.5854802088020144`*^9}, {3.585480504083839*^9, 
   3.5854805310292397`*^9}, {3.585480828413793*^9, 3.58548084492684*^9}, 
   3.5854810119248743`*^9, {3.5854812616869593`*^9, 3.585481292119055*^9}, 
   3.585481429615769*^9, {3.5854838619710693`*^9, 3.585483869149685*^9}, {
   3.585484991629231*^9, 3.585484994897126*^9}, {3.585485069519537*^9, 
   3.585485081124069*^9}, 3.5854851270822372`*^9, 3.585485204723464*^9, 
   3.585485426449129*^9, 3.58549547933384*^9, 3.585495513192103*^9, 
   3.58549563291991*^9, {3.5855010027686033`*^9, 3.585501024369157*^9}, 
   3.585501080824758*^9, 3.585501631051302*^9, 3.58550235139354*^9, {
   3.5856571730924187`*^9, 3.5856571923086348`*^9}, 3.585657247226851*^9, 
   3.58565737213406*^9, 3.5856575693203506`*^9, 3.5856577462240763`*^9, {
   3.585657826414122*^9, 3.585657884076195*^9}, 3.5856579148270683`*^9, 
   3.587798775352339*^9, {3.592286715489188*^9, 3.592286742847804*^9}, 
   3.592286780991713*^9, 3.592289779525482*^9, 3.592290296942675*^9, 
   3.592290495449703*^9, 3.592290629007367*^9, 3.592290832944779*^9, 
   3.592290985578616*^9, {3.592291035403165*^9, 3.592291061197184*^9}, 
   3.592291141540131*^9, 3.592291355784978*^9, {3.592291500782648*^9, 
   3.59229151327439*^9}, {3.5922915771894827`*^9, 3.5922916065827093`*^9}, {
   3.592291913365218*^9, 3.592291947344473*^9}, 3.592292335291814*^9, 
   3.5922928142035923`*^9, 3.592301638027232*^9, 3.592302726500683*^9, 
   3.592303127600919*^9, 3.592306500881908*^9, 3.592317514991899*^9, 
   3.5923175495160637`*^9, 3.5923175974843397`*^9, 3.592318436530422*^9, 
   3.592318797254775*^9, {3.592318884817946*^9, 3.592318913616189*^9}, 
   3.592318962382979*^9, 3.592319188817265*^9, 3.592319379881168*^9, 
   3.59533468549302*^9, 3.595334726420724*^9, 3.595422766014524*^9, 
   3.600171848878743*^9, 3.600171896747239*^9, 3.600172771813335*^9, 
   3.600175297194201*^9, 3.601099684826344*^9, {3.601099763985547*^9, 
   3.601099787812758*^9}, 3.601099826690124*^9, 3.601100103120957*^9, 
   3.601100547271966*^9, {3.6011006515400553`*^9, 3.6011006909186687`*^9}, 
   3.601100885337612*^9, 3.601101040969234*^9, 3.601101526413773*^9, 
   3.601101574965777*^9, 3.6011017389706793`*^9, 3.601128655583651*^9, 
   3.601128695665853*^9, {3.601128796548131*^9, 3.601128825268094*^9}, {
   3.601128947733292*^9, 3.601128957171373*^9}, {3.601189811683736*^9, 
   3.601189821058838*^9}, 3.60120627385065*^9, 3.602862401463297*^9, 
   3.6028650853876333`*^9, 3.602865123796536*^9, 3.602865186435557*^9, 
   3.60286549067487*^9, 3.602866887012306*^9, 3.6028669182178793`*^9, 
   3.602918622321424*^9, 3.602919047853869*^9, {3.60291910372561*^9, 
   3.60291913319932*^9}, 3.602919248222138*^9, 3.602919285976425*^9, 
   3.602920099061532*^9, {3.602920163426469*^9, 3.602920185726675*^9}, 
   3.602920226950211*^9, {3.6029202695884647`*^9, 3.602920296362204*^9}, 
   3.602930420076086*^9, {3.602930481041069*^9, 3.60293052370725*^9}, 
   3.602930557320788*^9, {3.6029305978993673`*^9, 3.6029306252523613`*^9}, 
   3.6029307726307583`*^9, 3.6029311088834553`*^9, 3.603543064798218*^9, 
   3.603543668015766*^9, 3.603543776507148*^9, 3.603543947613152*^9, 
   3.606477949497541*^9, 3.608794461479051*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalPhi]2", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[Phi]", "[", "4", "]"}], "+", 
       RowBox[{"I", " ", 
        RowBox[{"\[Phi]", "[", "5", "]"}]}]}], ")"}], "/", 
     RowBox[{"Sqrt", "[", "2", "]"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[Phi]", "[", "6", "]"}], "+", 
       RowBox[{"I", " ", 
        RowBox[{"\[Phi]", "[", "7", "]"}]}]}], ")"}], "/", 
     RowBox[{"Sqrt", "[", "2", "]"}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.549618764769656*^9, 3.5496187702269573`*^9}, {
  3.563002584479318*^9, 3.563002599228941*^9}, {3.566637273703445*^9, 
  3.566637282579597*^9}, {3.570779429412612*^9, 3.57077945600967*^9}, {
  3.570780291194472*^9, 3.570780308812855*^9}, {3.583126444200058*^9, 
  3.583126449305698*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"\[Phi]", "[", "4", "]"}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      RowBox[{"\[Phi]", "[", "5", "]"}]}]}], 
    SqrtBox["2"]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"\[Phi]", "[", "6", "]"}], "+", 
     RowBox[{"\[ImaginaryI]", " ", 
      RowBox[{"\[Phi]", "[", "7", "]"}]}]}], 
    SqrtBox["2"]]}], "}"}]], "Output",
 CellChangeTimes->{
  3.570780346601282*^9, 3.570782348054008*^9, 3.570801957347242*^9, 
   3.570802000985834*^9, 3.570802123118985*^9, 3.570802161506234*^9, 
   3.570802221488554*^9, 3.570802303318881*^9, 3.573370250771912*^9, {
   3.5771006746229258`*^9, 3.5771006832599087`*^9}, 3.577100911978389*^9, 
   3.577102192753706*^9, 3.5831268405206614`*^9, 3.5831271779989567`*^9, 
   3.583140376823208*^9, 3.583140446430168*^9, 3.583140517886548*^9, 
   3.583141545039342*^9, 3.583141904874168*^9, 3.583143620770966*^9, 
   3.583143650793816*^9, 3.583758932164872*^9, 3.5837643076903048`*^9, 
   3.5837645004901657`*^9, 3.583765328126136*^9, {3.585480040375993*^9, 
   3.585480057412497*^9}, {3.5854801491900463`*^9, 3.585480158956492*^9}, {
   3.585480200334222*^9, 3.5854802088321733`*^9}, {3.585480504111205*^9, 
   3.585480531060463*^9}, {3.585480828444807*^9, 3.585480844956789*^9}, 
   3.5854810120412188`*^9, {3.585481261714574*^9, 3.585481292149828*^9}, 
   3.5854814297723846`*^9, {3.585483861997044*^9, 3.5854838691717033`*^9}, {
   3.58548499165786*^9, 3.5854849949320173`*^9}, {3.5854850696677523`*^9, 
   3.585485081292769*^9}, 3.585485127239545*^9, 3.585485204874608*^9, 
   3.585485426472546*^9, 3.585495479364066*^9, 3.585495513317865*^9, 
   3.585495632947315*^9, {3.585501002791709*^9, 3.585501024490181*^9}, 
   3.585501080852581*^9, 3.58550163108727*^9, 3.585502351602419*^9, {
   3.5856571731250887`*^9, 3.585657192349846*^9}, 3.585657247263158*^9, 
   3.5856573722397614`*^9, 3.585657569359248*^9, 3.5856577462496233`*^9, {
   3.585657826569096*^9, 3.585657884222698*^9}, 3.5856579148669357`*^9, 
   3.5877987753701487`*^9, {3.5922867155571623`*^9, 3.592286742855411*^9}, 
   3.5922867809995728`*^9, 3.5922897795319977`*^9, 3.592290297016686*^9, 
   3.592290495456629*^9, 3.592290629014674*^9, 3.5922908329519043`*^9, 
   3.592290985585787*^9, {3.592291035410863*^9, 3.592291061262936*^9}, 
   3.592291141547061*^9, 3.592291355792604*^9, {3.5922915008550167`*^9, 
   3.592291513281971*^9}, {3.592291577257922*^9, 3.5922916065896177`*^9}, {
   3.592291913372854*^9, 3.59229194735224*^9}, 3.592292335298921*^9, 
   3.592292814276576*^9, 3.592301638034437*^9, 3.59230272650732*^9, 
   3.5923031276639214`*^9, 3.592306500889028*^9, 3.592317514997981*^9, 
   3.592317549588642*^9, 3.592317597554213*^9, 3.592318436537209*^9, 
   3.5923187972619953`*^9, {3.5923188848782187`*^9, 3.592318913684019*^9}, 
   3.5923189623909197`*^9, 3.592319188886386*^9, 3.592319379944664*^9, 
   3.5953346854995337`*^9, 3.595334726428186*^9, 3.595422766021031*^9, 
   3.600171848900126*^9, 3.600171896927001*^9, 3.600172771842885*^9, 
   3.600175297340617*^9, 3.601099684853609*^9, {3.601099764017561*^9, 
   3.6010997878455887`*^9}, 3.60109982684558*^9, 3.60110010314532*^9, 
   3.6011005473569183`*^9, {3.60110065166362*^9, 3.601100690942636*^9}, 
   3.601100885362234*^9, 3.601101041124538*^9, 3.601101526436262*^9, 
   3.601101575166892*^9, 3.601101739079707*^9, 3.601128655728709*^9, 
   3.6011286956976357`*^9, {3.601128796577694*^9, 3.601128825301324*^9}, {
   3.601128947794757*^9, 3.60112895719825*^9}, {3.601189811709688*^9, 
   3.601189821084585*^9}, 3.60120627397007*^9, 3.602862401552161*^9, 
   3.6028650854084997`*^9, 3.602865123828951*^9, 3.602865186459263*^9, 
   3.6028654908311977`*^9, 3.602866887101592*^9, 3.602866918251162*^9, 
   3.602918622347828*^9, 3.6029190478892384`*^9, {3.602919103762388*^9, 
   3.602919133309578*^9}, 3.602919248258003*^9, 3.6029192860958776`*^9, 
   3.60292009916962*^9, {3.6029201635924377`*^9, 3.6029201857587547`*^9}, 
   3.602920227081451*^9, {3.602920269621541*^9, 3.6029202964908047`*^9}, 
   3.6029304201009502`*^9, {3.60293048116297*^9, 3.602930523832642*^9}, 
   3.60293055742791*^9, {3.602930597931448*^9, 3.602930625280696*^9}, 
   3.602930772660665*^9, 3.602931108999029*^9, 3.603543064824531*^9, 
   3.60354366804629*^9, 3.60354377653876*^9, 3.603543947646139*^9, 
   3.606477949518879*^9, 3.608794461490162*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalPhi]2Dag", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[Phi]", "[", "4", "]"}], "-", 
       RowBox[{"I", " ", 
        RowBox[{"\[Phi]", "[", "5", "]"}]}]}], ")"}], "/", 
     RowBox[{"Sqrt", "[", "2", "]"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[Phi]", "[", "6", "]"}], "-", 
       RowBox[{"I", " ", 
        RowBox[{"\[Phi]", "[", "7", "]"}]}]}], ")"}], "/", 
     RowBox[{"Sqrt", "[", "2", "]"}]}]}], "}"}]}]], "Input",
 CellChangeTimes->{{3.549618781513886*^9, 3.5496187832577333`*^9}, {
  3.566637285796927*^9, 3.56663730892208*^9}, {3.570780342736515*^9, 
  3.570780352659683*^9}, {3.583126453380465*^9, 3.5831264592357273`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"\[Phi]", "[", "4", "]"}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      RowBox[{"\[Phi]", "[", "5", "]"}]}]}], 
    SqrtBox["2"]], ",", 
   FractionBox[
    RowBox[{
     RowBox[{"\[Phi]", "[", "6", "]"}], "-", 
     RowBox[{"\[ImaginaryI]", " ", 
      RowBox[{"\[Phi]", "[", "7", "]"}]}]}], 
    SqrtBox["2"]]}], "}"}]], "Output",
 CellChangeTimes->{
  3.549618784025077*^9, 3.549619416777308*^9, 3.549620984013175*^9, 
   3.549621063422876*^9, {3.549621154327694*^9, 3.5496211796874437`*^9}, 
   3.5496216550449667`*^9, 3.549621706315928*^9, 3.5497068461074057`*^9, 
   3.549711878211996*^9, 3.552302631999094*^9, 3.552302732540604*^9, 
   3.552302812743764*^9, 3.552305502847011*^9, 3.552842049484744*^9, 
   3.555738441206912*^9, 3.555741444622259*^9, 3.570347593958298*^9, 
   3.570780353408888*^9, 3.570782348288912*^9, 3.570801957439808*^9, 
   3.570802001049767*^9, 3.570802123172003*^9, 3.570802161525991*^9, 
   3.570802221504252*^9, 3.570802303441547*^9, 3.573370250872563*^9, {
   3.577100674651019*^9, 3.577100683359651*^9}, 3.577100912221837*^9, 
   3.577102192768567*^9, 3.5831268407790318`*^9, 3.583127178028216*^9, 
   3.5831403771192923`*^9, 3.58314044657303*^9, 3.583140518169594*^9, 
   3.583141545332766*^9, 3.583141905022249*^9, 3.583143620877715*^9, 
   3.5831436509137983`*^9, 3.583758932470837*^9, 3.5837643080253572`*^9, 
   3.583764500845117*^9, 3.583765328400403*^9, {3.5854800404014177`*^9, 
   3.585480057442377*^9}, {3.585480149218944*^9, 3.585480158979529*^9}, {
   3.5854802003602457`*^9, 3.585480208863248*^9}, {3.585480504135206*^9, 
   3.5854805310882893`*^9}, {3.5854808284757147`*^9, 3.585480845088469*^9}, 
   3.585481012068084*^9, {3.585481261737444*^9, 3.585481292180888*^9}, 
   3.585481429805312*^9, {3.5854838620764847`*^9, 3.585483869192624*^9}, {
   3.5854849916864557`*^9, 3.5854849949657583`*^9}, {3.58548506970336*^9, 
   3.585485081470427*^9}, 3.58548512740105*^9, 3.585485205039547*^9, 
   3.585485426498279*^9, 3.5854954793910427`*^9, 3.58549551334934*^9, 
   3.585495633050075*^9, {3.585501002815011*^9, 3.585501024520792*^9}, 
   3.5855010809696617`*^9, 3.585501631115972*^9, 3.585502351747164*^9, {
   3.585657173150609*^9, 3.585657192381917*^9}, 3.585657247294751*^9, 
   3.585657372458053*^9, 3.585657569390242*^9, 3.585657746279716*^9, {
   3.585657826593946*^9, 3.585657884265794*^9}, 3.585657914905835*^9, 
   3.587798775386977*^9, {3.592286715621936*^9, 3.592286742863041*^9}, 
   3.592286781007265*^9, 3.592289779538368*^9, 3.592290297087846*^9, 
   3.5922904954632807`*^9, 3.592290629021806*^9, 3.592290832959344*^9, 
   3.5922909855929623`*^9, {3.592291035418449*^9, 3.592291061320561*^9}, 
   3.5922911415547028`*^9, 3.5922913558001757`*^9, {3.592291500925888*^9, 
   3.592291513289556*^9}, {3.592291577326441*^9, 3.5922916065964518`*^9}, {
   3.592291913380437*^9, 3.5922919473597717`*^9}, 3.592292335305978*^9, 
   3.592292814344924*^9, 3.592301638041534*^9, 3.592302726513876*^9, 
   3.59230312772654*^9, 3.5923065008960238`*^9, 3.592317515003893*^9, 
   3.5923175496523857`*^9, 3.592317597622089*^9, 3.5923184366064167`*^9, 
   3.5923187972703342`*^9, {3.592318884945519*^9, 3.5923189137470207`*^9}, 
   3.592318962398006*^9, 3.592319188960286*^9, 3.5923193800135317`*^9, 
   3.595334685505987*^9, 3.595334726434881*^9, 3.5954227660557547`*^9, 
   3.600171848921348*^9, 3.600171897116761*^9, 3.600172771966998*^9, 
   3.600175297372366*^9, 3.601099684963904*^9, {3.601099764138915*^9, 
   3.6010997878781967`*^9}, 3.6010998268740683`*^9, 3.601100103166974*^9, 
   3.6011005474423733`*^9, {3.601100651686995*^9, 3.601100690965967*^9}, 
   3.601100885382854*^9, 3.6011010412658243`*^9, 3.601101526458682*^9, 
   3.601101575491089*^9, 3.601101739185611*^9, 3.601128655868853*^9, 
   3.601128695839937*^9, {3.601128796601387*^9, 3.601128825332206*^9}, {
   3.601128947857491*^9, 3.601128957227717*^9}, {3.601189811736405*^9, 
   3.601189821113378*^9}, 3.6012062740822287`*^9, 3.6028624016340847`*^9, 
   3.602865085432849*^9, 3.602865123852215*^9, 3.602865186482637*^9, 
   3.602865490967408*^9, 3.602866887124653*^9, 3.602866918283489*^9, 
   3.602918622373682*^9, 3.602919047918812*^9, {3.6029191038709793`*^9, 
   3.602919133338463*^9}, 3.6029192482858057`*^9, 3.602919286128763*^9, 
   3.6029200991944027`*^9, {3.6029201636282177`*^9, 3.602920185787462*^9}, 
   3.602920227116317*^9, {3.602920269646873*^9, 3.602920296522633*^9}, 
   3.602930420127851*^9, {3.602930481275511*^9, 3.602930523952718*^9}, 
   3.602930557457255*^9, {3.602930597963417*^9, 3.602930625307667*^9}, 
   3.60293077269213*^9, 3.6029311091085987`*^9, 3.6035430648548813`*^9, 
   3.6035436680818167`*^9, 3.60354377657406*^9, 3.603543947672491*^9, 
   3.6064779495385103`*^9, 3.6087944615126057`*^9}]
}, Open  ]],

Cell["\<\
The Vscalar quantity always has to be defined. This is the scalar potential \
that will be passed to the cpp code through the input file generated by this \
notebook. This is a 2HDM example!\
\>", "Text",
 CellChangeTimes->{{3.58547972633412*^9, 3.5854797380006313`*^9}, {
  3.601101910297043*^9, 3.601101918911748*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Vscalar", "=", 
  RowBox[{"ComplexExpand", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"L", "[", "0", "]"}], 
     RowBox[{"\[CapitalPhi]1Dag", ".", "\[CapitalPhi]1"}]}], "+", 
    RowBox[{
     RowBox[{"L", "[", "1", "]"}], " ", 
     RowBox[{"\[CapitalPhi]2Dag", ".", "\[CapitalPhi]2"}]}], "-", 
    RowBox[{
     RowBox[{"L", "[", "7", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[CapitalPhi]1Dag", ".", "\[CapitalPhi]2"}], "+", 
       RowBox[{"\[CapitalPhi]2Dag", ".", "\[CapitalPhi]1"}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"L", "[", "3", "]"}], "/", "2"}], " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"\[CapitalPhi]1Dag", ".", "\[CapitalPhi]1"}], ")"}], "^", 
      "2"}]}], " ", "+", 
    RowBox[{
     RowBox[{
      RowBox[{"L", "[", "4", "]"}], " ", "/", "2"}], " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"\[CapitalPhi]2Dag", ".", "\[CapitalPhi]2"}], ")"}], "^", 
      "2"}]}], "+", 
    RowBox[{
     RowBox[{"L", "[", "5", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{"\[CapitalPhi]1Dag", ".", "\[CapitalPhi]1"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"\[CapitalPhi]2Dag", ".", "\[CapitalPhi]2"}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"L", "[", "6", "]"}], 
     RowBox[{"(", 
      RowBox[{"\[CapitalPhi]2Dag", ".", "\[CapitalPhi]1"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"\[CapitalPhi]1Dag", ".", "\[CapitalPhi]2"}], ")"}]}], "+", 
    RowBox[{
     RowBox[{
      RowBox[{"L", "[", "2", "]"}], "/", "2"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"\[CapitalPhi]1Dag", ".", "\[CapitalPhi]2"}], ")"}], "^", 
        "2"}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"\[CapitalPhi]2Dag", ".", "\[CapitalPhi]1"}], ")"}], "^", 
        "2"}]}], ")"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.534144310207885*^9, 3.5341444506966047`*^9}, {
   3.534144490509598*^9, 3.53414458883617*^9}, {3.534144624519189*^9, 
   3.534144666441533*^9}, {3.534178088516856*^9, 3.534178111806834*^9}, {
   3.534178147673089*^9, 3.5341781921278267`*^9}, {3.534178224906232*^9, 
   3.534178242164892*^9}, {3.5342465770748453`*^9, 3.5342465879329967`*^9}, {
   3.53424662105466*^9, 3.5342466564110603`*^9}, {3.53510999916111*^9, 
   3.535110001047953*^9}, {3.53511005129663*^9, 3.535110051792281*^9}, {
   3.535110277667881*^9, 3.535110290667621*^9}, {3.535110760050255*^9, 
   3.535110761128691*^9}, 3.535111722829476*^9, {3.535112079559593*^9, 
   3.535112145279432*^9}, {3.535112452325189*^9, 3.535112534267795*^9}, {
   3.535112896297239*^9, 3.535112912268751*^9}, {3.535115195852663*^9, 
   3.535115308267053*^9}, {3.535784382495168*^9, 3.535784383535134*^9}, {
   3.535805240382502*^9, 3.535805241186414*^9}, {3.536384713321053*^9, 
   3.5363847140485077`*^9}, {3.537020326846611*^9, 3.53702034303214*^9}, {
   3.537020416238227*^9, 3.5370204701153297`*^9}, 3.537189581054373*^9, 
   3.538305507421482*^9, {3.547906780982019*^9, 3.547906795151812*^9}, {
   3.5484835535811167`*^9, 3.548483554435574*^9}, {3.5495536372529383`*^9, 
   3.54955368415438*^9}, {3.5495558345272703`*^9, 3.5495558345724993`*^9}, 
   3.549555984248444*^9, 3.549556019979096*^9, {3.549618831668489*^9, 
   3.549618903965797*^9}, {3.549620947291133*^9, 3.549620954260638*^9}, {
   3.549706769625893*^9, 3.549706774600843*^9}, {3.563002618635407*^9, 
   3.563002639581672*^9}, {3.563003917132162*^9, 3.563003968937449*^9}, {
   3.563005825040492*^9, 3.563005866097913*^9}, {3.566637402030693*^9, 
   3.566637402660519*^9}, {3.566637469465615*^9, 3.566637476736376*^9}, {
   3.566637516045632*^9, 3.566637602565646*^9}, {3.566637637466981*^9, 
   3.5666376384839*^9}, {3.56663768016229*^9, 3.566637696295292*^9}, {
   3.566637741054867*^9, 3.566637768054602*^9}, {3.566637829628353*^9, 
   3.566637855787619*^9}, {3.566644882033549*^9, 3.566644902937996*^9}, {
   3.566644998329653*^9, 3.566645000939214*^9}, 3.566646094522893*^9, 
   3.566646139462853*^9, 3.566646182418709*^9, 3.566646279403314*^9, {
   3.566646322090582*^9, 3.566646362101699*^9}, {3.583126462629496*^9, 
   3.5831264846759377`*^9}, {3.5854794846064167`*^9, 3.585479520218713*^9}, 
   3.585479720778413*^9, {3.5922867069295893`*^9, 3.592286709117131*^9}, {
   3.5922883937106123`*^9, 3.592288396789113*^9}, {3.5922896764076138`*^9, 
   3.592289678131303*^9}, {3.601101265596266*^9, 3.6011012802456627`*^9}, {
   3.601101505875526*^9, 3.6011015059450397`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "0", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "0", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "8"], " ", 
   RowBox[{"L", "[", "3", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "0", "]"}], "4"]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "0", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "1", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "3", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "0", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "1", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "8"], " ", 
   RowBox[{"L", "[", "3", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "1", "]"}], "4"]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "0", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "2", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "3", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "0", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "2", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "3", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "1", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "2", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "8"], " ", 
   RowBox[{"L", "[", "3", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "2", "]"}], "4"]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "0", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "3", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "3", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "0", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "3", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "3", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "1", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "3", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "3", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "2", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "3", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "8"], " ", 
   RowBox[{"L", "[", "3", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "3", "]"}], "4"]}], "-", 
  RowBox[{
   RowBox[{"L", "[", "7", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "0", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "4", "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "1", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "4", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "2", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "0", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "4", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "5", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "0", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "4", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "6", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "0", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "4", "]"}], "2"]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "2", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "1", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "4", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "5", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "1", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "4", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "6", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "1", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "4", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "5", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "2", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "4", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "5", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "3", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "4", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "8"], " ", 
   RowBox[{"L", "[", "4", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "4", "]"}], "4"]}], "-", 
  RowBox[{
   RowBox[{"L", "[", "7", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "1", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "5", "]"}]}], "+", 
  RowBox[{
   RowBox[{"L", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "0", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "1", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "4", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "5", "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "1", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "5", "]"}], "2"]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "2", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "0", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "5", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "5", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "0", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "5", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "6", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "0", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "5", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "2", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "1", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "5", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "5", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "1", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "5", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "6", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "1", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "5", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "5", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "2", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "5", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "5", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "3", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "5", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "4", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "4", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "5", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "8"], " ", 
   RowBox[{"L", "[", "4", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "5", "]"}], "4"]}], "-", 
  RowBox[{
   RowBox[{"L", "[", "7", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "6", "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "0", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "4", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "6", "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "6", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "0", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "4", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "6", "]"}]}], "-", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "1", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "3", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "4", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "6", "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "6", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "1", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "3", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "4", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "6", "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "1", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "5", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "6", "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "6", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "1", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "5", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "6", "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "0", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "3", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "5", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "6", "]"}]}], "-", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "6", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "0", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "3", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "5", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "6", "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "1", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "6", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "5", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "0", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "6", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "5", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "1", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "6", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "2", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "2", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "6", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "5", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "2", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "6", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "6", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "2", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "6", "]"}], "2"]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "2", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "3", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "6", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "5", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "3", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "6", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "6", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "3", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "6", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "4", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "4", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "6", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "4", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "5", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "6", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "8"], " ", 
   RowBox[{"L", "[", "4", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "6", "]"}], "4"]}], "-", 
  RowBox[{
   RowBox[{"L", "[", "7", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "3", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "7", "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "1", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "4", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "7", "]"}]}], "-", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "6", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "1", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "4", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "7", "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "0", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "3", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "4", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "7", "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "6", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "0", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "3", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "4", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "7", "]"}]}], "-", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "0", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "5", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "7", "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "6", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "0", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "5", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "7", "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "1", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "3", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "5", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "7", "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "6", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "1", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "3", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "5", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "7", "]"}]}], "+", 
  RowBox[{
   RowBox[{"L", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "2", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "3", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "6", "]"}], " ", 
   RowBox[{"\[Phi]", "[", "7", "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"L", "[", "1", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "7", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "5", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "0", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "7", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "5", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "1", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "7", "]"}], "2"]}], "-", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "2", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "2", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "7", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "5", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "2", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "7", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "6", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "2", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "7", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "2", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "3", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "7", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "5", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "3", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "7", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "6", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "3", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "7", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "4", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "4", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "7", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "4", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "5", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "7", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"L", "[", "4", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "6", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "7", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "8"], " ", 
   RowBox[{"L", "[", "4", "]"}], " ", 
   SuperscriptBox[
    RowBox[{"\[Phi]", "[", "7", "]"}], "4"]}]}]], "Output",
 CellChangeTimes->{
  3.549553688644093*^9, 3.549554039687963*^9, 3.549555530601707*^9, 
   3.549555848034932*^9, {3.5495559974421377`*^9, 3.549556023744195*^9}, 
   3.549566949367874*^9, 3.5496006436360598`*^9, 3.5496006912214403`*^9, 
   3.549618907268854*^9, 3.5496194168355007`*^9, 3.5496209841571417`*^9, 
   3.549621063497464*^9, {3.5496211544701147`*^9, 3.54962117981526*^9}, 
   3.5496216551061783`*^9, 3.549621706388008*^9, 3.549706846546579*^9, 
   3.5497118783307123`*^9, 3.5523026321262503`*^9, 3.552302732594841*^9, 
   3.552302812801145*^9, 3.552305502904253*^9, 3.552842049532589*^9, 
   3.555738441253096*^9, 3.55574144469486*^9, 3.570347594470019*^9, 
   3.570780359979332*^9, 3.570782348939485*^9, 3.5708019576759*^9, 
   3.570802001141002*^9, 3.570802123576994*^9, 3.570802161608792*^9, 
   3.57080222159209*^9, 3.570802303763793*^9, 3.573370251055904*^9, {
   3.5771006749764*^9, 3.5771006834427433`*^9}, 3.577100912709074*^9, 
   3.577102193120843*^9, 3.5831268411420383`*^9, 3.583127178175931*^9, 
   3.5831403775850286`*^9, 3.5831404468537607`*^9, 3.583140518458955*^9, 
   3.5831415456490717`*^9, 3.5831419057214527`*^9, 3.58314362101892*^9, 
   3.5831436510472527`*^9, 3.5837589331147003`*^9, 3.58376430871397*^9, 
   3.5837645014029837`*^9, 3.583765328929957*^9, {3.5854800406797523`*^9, 
   3.5854800576124687`*^9}, {3.585480149402946*^9, 3.5854801591232157`*^9}, {
   3.58548020054023*^9, 3.585480209040914*^9}, {3.5854805043043747`*^9, 
   3.585480531237626*^9}, {3.58548082863558*^9, 3.585480845374082*^9}, 
   3.585481012265594*^9, {3.58548126186736*^9, 3.585481292315051*^9}, 
   3.58548143004659*^9, {3.585483862181725*^9, 3.5854838692892437`*^9}, {
   3.5854849918280497`*^9, 3.5854849950971317`*^9}, {3.5854850698564577`*^9, 
   3.585485081664442*^9}, 3.58548512759569*^9, 3.585485205223382*^9, 
   3.5854854266513147`*^9, 3.5854954795318117`*^9, 3.585495513493376*^9, 
   3.585495633175219*^9, {3.5855010029632263`*^9, 3.5855010246547947`*^9}, 
   3.5855010811014223`*^9, 3.585501631396014*^9, 3.5855023519424877`*^9, {
   3.5856571733180323`*^9, 3.585657192647743*^9}, 3.585657247514949*^9, 
   3.585657372941388*^9, 3.585657569673048*^9, 3.585657746516334*^9, {
   3.5856578267411547`*^9, 3.585657884539351*^9}, 3.585657915281763*^9, 
   3.587798775557272*^9, {3.592286715829852*^9, 3.59228674296068*^9}, 
   3.5922867811107893`*^9, 3.592289779575858*^9, 3.592290297155944*^9, 
   3.5922904955423803`*^9, 3.592290629082149*^9, 3.5922908330160522`*^9, 
   3.592290985650258*^9, {3.5922910354771147`*^9, 3.5922910613871927`*^9}, 
   3.592291141618182*^9, 3.592291355870722*^9, {3.592291500996108*^9, 
   3.592291513358506*^9}, {3.592291577398048*^9, 3.592291606656454*^9}, {
   3.592291913438218*^9, 3.59229194740633*^9}, 3.592292335363076*^9, 
   3.592292814408105*^9, 3.592301638104163*^9, 3.592302726574204*^9, 
   3.592303127805565*^9, 3.592306500961237*^9, 3.5923175150398817`*^9, 
   3.592317549754422*^9, 3.5923175976854677`*^9, 3.592318436681068*^9, 
   3.59231879734595*^9, {3.592318885030579*^9, 3.592318913808695*^9}, 
   3.592318962464745*^9, 3.592319189032867*^9, 3.5923193800838118`*^9, 
   3.5953346855983877`*^9, 3.595334726536654*^9, 3.595422766226864*^9, 
   3.600171849182041*^9, 3.60017189730976*^9, 3.6001727721012793`*^9, 
   3.6001752975236263`*^9, 3.6010996851966267`*^9, {3.6010997642801723`*^9, 
   3.601099788102736*^9}, 3.601099827059338*^9, 3.601100103315907*^9, 
   3.601100547650895*^9, {3.601100651817656*^9, 3.601100691081542*^9}, 
   3.601100885535234*^9, 3.601101041494817*^9, 3.601101526593111*^9, 
   3.601101575935257*^9, 3.601101739338663*^9, 3.601128656100157*^9, 
   3.601128696051106*^9, {3.601128796761669*^9, 3.601128825422241*^9}, {
   3.601128947948361*^9, 3.601128957366045*^9}, {3.601189812081645*^9, 
   3.601189821235915*^9}, 3.6012062742323847`*^9, 3.6028624018644342`*^9, 
   3.602865085564761*^9, 3.6028651239670973`*^9, 3.602865186614715*^9, 
   3.6028654911010923`*^9, 3.602866887217909*^9, 3.6028669184287643`*^9, 
   3.602918622526065*^9, 3.602919048055045*^9, {3.60291910400264*^9, 
   3.602919133463284*^9}, 3.602919248453247*^9, 3.602919286261615*^9, 
   3.602920099351531*^9, {3.6029201637981367`*^9, 3.6029201859403067`*^9}, 
   3.602920227247141*^9, {3.602920269799995*^9, 3.602920296711775*^9}, 
   3.60293042026126*^9, {3.602930481437416*^9, 3.602930524108444*^9}, 
   3.602930557566765*^9, {3.602930598073037*^9, 3.602930625446906*^9}, 
   3.6029307727944517`*^9, 3.6029311092517643`*^9, 3.603543065198883*^9, 
   3.6035436682273912`*^9, 3.603543776697798*^9, 3.6035439478050423`*^9, 
   3.606477949756673*^9, 3.608794461797879*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["VEV Ranges (in case a simple hypercubic box is chosen)", "Subsection",
 CellChangeTimes->{{3.546865685914077*^9, 3.5468656870094137`*^9}, {
  3.585482911811902*^9, 3.5854829161180058`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "Initializing", " ", "all", " ", "ranges", " ", "to", " ", "zero"}], " ", 
   "*)"}], "\[IndentingNewLine]", 
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"i", "=", "0"}], ",", 
    RowBox[{"i", "<", "Nreal"}], ",", 
    RowBox[{"++", "i"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]Min", "[", "i", "]"}], "=", "0"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"\[Phi]Max", "[", "i", "]"}], "=", "0"}], ";"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.583126509647599*^9, 3.583126516495706*^9}, {
  3.585482829270853*^9, 3.5854828627618647`*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{"Now", " ", "defining", " ", "the", " ", "non"}], "-", 
   RowBox[{"zero", " ", "ranges", " ", 
    RowBox[{"WARNING", ":", " ", 
     RowBox[{
     "In", " ", "this", " ", "example", " ", "This", " ", "is", " ", 
      "overriden", " ", "by", " ", "the", " ", "parametrised", " ", "VEVs", 
      " ", "below"}]}]}]}], "*)"}]], "Input",
 CellChangeTimes->{{3.585482848181301*^9, 3.585482860081266*^9}, {
  3.601101135002758*^9, 3.601101159106327*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"\[Phi]Min", "[", "2", "]"}], "=", "0"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[Phi]Max", "[", "2", "]"}], "=", "246"}], ";"}]}], "Input",
 CellChangeTimes->{{3.549640440750679*^9, 3.54964044199866*^9}, {
   3.549640552214943*^9, 3.5496405532412663`*^9}, {3.5830840323108587`*^9, 
   3.583084033277264*^9}, {3.583084079528722*^9, 3.5830840809419813`*^9}, 
   3.5831265684527063`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"\[Phi]Min", "[", "6", "]"}], "=", "0"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[Phi]Max", "[", "6", "]"}], "=", "246"}], ";"}]}], "Input",
 CellChangeTimes->{{3.5497086039551287`*^9, 3.549708605027857*^9}, {
   3.552574817509997*^9, 3.552574817921376*^9}, {3.555740313591342*^9, 
   3.5557403138962927`*^9}, 3.5830741737471848`*^9, {3.583084043748694*^9, 
   3.58308404510778*^9}, {3.583084088600975*^9, 3.583084090382546*^9}, {
   3.583126563573991*^9, 3.583126571281637*^9}}],

Cell[CellGroupData[{

Cell["\<\
Parametrized VEV ranges (optional --  ON if  NparamsVEVs > 0)\
\>", "Subsubsection",
 CellChangeTimes->{{3.5854829702489777`*^9, 3.585483004325163*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Initializing", " ", "all", " ", "ranges", " ", "to", " ", "zero"}],
   " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"i", "=", "0"}], ",", 
   RowBox[{"i", "<", "NparamsVEVs"}], ",", 
   RowBox[{"++", "i"}], ",", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]ParMin", "[", "i", "]"}], "=", "0"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"\[Phi]ParMax", "[", "i", "]"}], "=", "0"}], ";"}]}], 
  "\[IndentingNewLine]", "]"}]], "Input",
 CellChangeTimes->{{3.583126581349114*^9, 3.5831266062864437`*^9}, 
   3.585478893972082*^9}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{
   "Then", " ", "initialize", " ", "ranges", " ", "for", " ", "parameters", 
    " ", "used", " ", "in", " ", "the", " ", "parametrization", " ", 
    "defined", " ", "in", " ", "the", " ", "user"}], "-", 
   RowBox[{
   "defined", " ", "function", " ", "\"\<MyPhiParametrization\>\"", " ", "in",
     " ", 
    RowBox[{"ScannerSUser", ".", "cpp"}]}]}], " ", " ", "*)"}]], "Input",
 CellChangeTimes->{{3.5854830348626966`*^9, 3.585483081032977*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"\[Phi]ParMin", "[", "0", "]"}], "=", "0.01"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[Phi]ParMax", "[", "0", "]"}], "=", 
   RowBox[{
    RowBox[{"Pi", "/", "2"}], "-", "0.01"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.549640419086753*^9, 3.549640426375087*^9}, {
   3.549640544747589*^9, 3.549640545463179*^9}, {3.5830718657475977`*^9, 
   3.583071880718238*^9}, {3.583071916677647*^9, 3.583071918359364*^9}, {
   3.583071955764485*^9, 3.5830719559831743`*^9}, {3.583072728597533*^9, 
   3.583072743941885*^9}, {3.58308406561486*^9, 3.5830840673143587`*^9}, {
   3.5831198193998013`*^9, 3.583119833680192*^9}, {3.583126620567246*^9, 
   3.5831266357285147`*^9}, {3.583758520617651*^9, 3.583758522844577*^9}, 
   3.592286549382646*^9, {3.5922865817052107`*^9, 3.5922865818639183`*^9}, 
   3.600171843525477*^9, {3.6001718905540447`*^9, 3.600171892245483*^9}, {
   3.600172762161302*^9, 3.6001727682885923`*^9}, {3.600175278557181*^9, 
   3.6001752843077707`*^9}, {3.602862393716214*^9, 3.602862396470481*^9}, {
   3.602865483407284*^9, 3.602865483990099*^9}, {3.6028655172300777`*^9, 
   3.602865522389296*^9}, {3.602918608852469*^9, 3.602918616913185*^9}, {
   3.602920088396984*^9, 3.602920095369608*^9}, {3.602920139406083*^9, 
   3.602920211853394*^9}, 3.6029304169310904`*^9, 3.6029306195519323`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Parametrized Mixing matrix ranges (optional --  ON if  NparamsMix> 0)\
\>", "Subsubsection",
 CellChangeTimes->{{3.5854829702489777`*^9, 3.585483004325163*^9}, {
   3.585500564066173*^9, 3.585500571760414*^9}, 3.601128563762486*^9}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{"Initializing", " ", "all", " ", "ranges", " ", "to", " ", "zero"}],
   " ", "*)"}]], "Input"],

Cell[BoxData[
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"i", "=", "0"}], ",", 
   RowBox[{"i", "<", "NparamsMix"}], ",", 
   RowBox[{"++", "i"}], ",", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"MixParMin", "[", "i", "]"}], "=", "0"}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"MixParMax", "[", "i", "]"}], "=", "0"}], ";"}]}], 
  "\[IndentingNewLine]", "]"}]], "Input",
 CellChangeTimes->{{3.583126581349114*^9, 3.5831266062864437`*^9}, 
   3.585478893972082*^9, {3.585500575180869*^9, 3.585500581835403*^9}, 
   3.601128563776215*^9}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
   RowBox[{
   "Then", " ", "initialize", " ", "ranges", " ", "for", " ", "parameters", 
    " ", "used", " ", "in", " ", "the", " ", "parametrization", " ", 
    "defined", " ", "in", " ", "the", " ", "user"}], "-", 
   RowBox[{
   "defined", " ", "function", " ", "\"\<MyPhiParametrization\>\"", " ", "in",
     " ", 
    RowBox[{"ScannerSUser", ".", "cpp"}]}]}], " ", " ", "*)"}]], "Input",
 CellChangeTimes->{{3.5854830348626966`*^9, 3.585483081032977*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"MixParMin", "[", "0", "]"}], "=", "0.231"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"MixParMax", "[", "0", "]"}], "=", "0.231"}], ";"}]}], "Input",
 CellChangeTimes->{{3.549640419086753*^9, 3.549640426375087*^9}, {
   3.549640544747589*^9, 3.549640545463179*^9}, {3.5830718657475977`*^9, 
   3.583071880718238*^9}, {3.583071916677647*^9, 3.583071918359364*^9}, {
   3.583071955764485*^9, 3.5830719559831743`*^9}, {3.583072728597533*^9, 
   3.583072743941885*^9}, {3.58308406561486*^9, 3.5830840673143587`*^9}, {
   3.5831198193998013`*^9, 3.583119833680192*^9}, {3.583126620567246*^9, 
   3.5831266357285147`*^9}, {3.583758520617651*^9, 3.583758522844577*^9}, {
   3.58550058811054*^9, 3.5855005912346077`*^9}, {3.5855016275648623`*^9, 
   3.585501627770669*^9}, 3.59228659249189*^9, {3.6035430576167803`*^9, 
   3.603543061778462*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Couplings and masses Ranges (always checked)", "Subsection",
 CellChangeTimes->{{3.585483121735176*^9, 3.585483194741048*^9}}],

Cell[CellGroupData[{

Cell["Coupling ranges", "Subsubsection",
 CellChangeTimes->{{3.585483247691101*^9, 3.585483250834515*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"m11", "^", "2"}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"LMin", "[", "0", "]"}], "=", 
     RowBox[{"-", 
      RowBox[{"10", "^", "9"}]}]}], ";"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"LMax", "[", "0", "]"}], "=", 
     RowBox[{"10", "^", "9"}]}], ";"}], "\n", 
   RowBox[{"(*", " ", 
    RowBox[{"m22", "^", "2"}], "*)"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"LMin", "[", "1", "]"}], "=", 
     RowBox[{"-", 
      RowBox[{"10", "^", "9"}]}]}], ";"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"LMax", "[", "1", "]"}], "=", 
     RowBox[{"10", "^", "9"}]}], ";"}], "\n", 
   RowBox[{"(*", " ", 
    RowBox[{"m12", "^", "2"}], "*)"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"LMin", "[", "7", "]"}], "=", 
     RowBox[{"-", 
      RowBox[{
       RowBox[{"(", "50.", ")"}], "^", "2"}]}]}], ";"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"LMax", "[", "7", "]"}], "=", 
     RowBox[{
      RowBox[{"(", "500.", ")"}], "^", "2"}]}], ";"}], "\n", "  ", 
   RowBox[{"(*", " ", "lambda1", " ", "*)"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"LMin", "[", "3", "]"}], "=", 
     RowBox[{"-", "10"}]}], ";"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"LMax", "[", "3", "]"}], "=", " ", "10"}], ";"}], "\n", "  ", 
   RowBox[{"(*", " ", "lambda2", " ", "*)"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"LMin", "[", "4", "]"}], "=", 
     RowBox[{"-", "10"}]}], ";"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"LMax", "[", "4", "]"}], "=", "10"}], ";"}], "\n", "  ", 
   RowBox[{"(*", " ", "lambda3", " ", "*)"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"LMin", "[", "5", "]"}], "=", 
     RowBox[{"-", "10"}]}], ";"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"LMax", "[", "5", "]"}], "=", " ", "10"}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.546865971934822*^9, 3.546866141749364*^9}, {
   3.54736590838002*^9, 3.547365950861902*^9}, {3.5495537902842817`*^9, 
   3.54955382098479*^9}, {3.549553853679388*^9, 3.549553982174777*^9}, {
   3.549566937249063*^9, 3.5495669446538687`*^9}, {3.549618978213484*^9, 
   3.549619137635911*^9}, {3.549619179935617*^9, 3.549619290523549*^9}, {
   3.5496209612335463`*^9, 3.549620961646048*^9}, {3.54970676170907*^9, 
   3.5497067633455143`*^9}, {3.552302726898242*^9, 3.552302728204114*^9}, {
   3.552302799556973*^9, 3.552302802056004*^9}, {3.555736872697688*^9, 
   3.555736887503777*^9}, {3.555738408882618*^9, 3.5557384285287857`*^9}, {
   3.555741414612959*^9, 3.5557414335266848`*^9}, {3.568360679305075*^9, 
   3.568360740060766*^9}, {3.570801886222523*^9, 3.570801926361846*^9}, {
   3.583126685576461*^9, 3.5831267223333273`*^9}, {3.585483228767868*^9, 
   3.585483229855309*^9}, {3.5922866186320972`*^9, 3.592286629408358*^9}, {
   3.5922904739570417`*^9, 3.592290480022092*^9}, {3.592290620418498*^9, 
   3.592290621866108*^9}, {3.5922908226248713`*^9, 3.592290823761899*^9}, {
   3.592290976324239*^9, 3.592291029466158*^9}, {3.592291130251649*^9, 
   3.592291134113763*^9}, {3.59229144332436*^9, 3.59229145341006*^9}, {
   3.592291572735766*^9, 3.5922916041115913`*^9}, {3.592291679468391*^9, 
   3.592291684677725*^9}, {3.592291864597876*^9, 3.592291877475593*^9}, {
   3.592292311170786*^9, 3.5922923265688868`*^9}, {3.600171788685131*^9, 
   3.600171788934499*^9}, 3.6001752674372377`*^9, {3.601101506255224*^9, 
   3.601101506379027*^9}, {3.601101729194911*^9, 3.6011017328699093`*^9}, {
   3.602865053295662*^9, 3.602865116341131*^9}, {3.602865167184185*^9, 
   3.6028651726235847`*^9}, {3.602866864564028*^9, 3.602866865887988*^9}, {
   3.602866903685298*^9, 3.6028669058636417`*^9}, {3.602919239064891*^9, 
   3.602919242266983*^9}, {3.602930477321211*^9, 3.602930555017572*^9}, {
   3.6029305947839727`*^9, 3.6029305949362383`*^9}, {3.602930762803884*^9, 
   3.602930767707191*^9}, {3.602931099875461*^9, 3.6029311050211763`*^9}}],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", "  ", 
  RowBox[{"(*", " ", "lambda4", " ", "*)"}], "\n", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"LMin", "[", "6", "]"}], "=", 
     RowBox[{"-", "10"}]}], ";"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"LMax", "[", "6", "]"}], "=", " ", "10"}], ";"}], "\n", "  ", 
   RowBox[{"(*", " ", "lambda5", " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"LMin", "[", "2", "]"}], "=", 
     RowBox[{"-", "10"}]}], ";"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"LMax", "[", "2", "]"}], "=", " ", "10"}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.568360748277626*^9, 3.568360772398319*^9}, {
  3.570801933871536*^9, 3.570801940909851*^9}, {3.5831267246278677`*^9, 
  3.583126737494256*^9}, {3.592290623528945*^9, 3.5922906249818153`*^9}, {
  3.592291049866562*^9, 3.5922910519886923`*^9}, {3.592291473637553*^9, 
  3.592291475785809*^9}, {3.592291882169922*^9, 3.592291890006777*^9}, {
  3.5922923066224623`*^9, 3.59229230956945*^9}, {3.601101506396885*^9, 
  3.60110150641861*^9}, {3.602865063101205*^9, 3.602865066405932*^9}, {
  3.6029192433158617`*^9, 3.602919246331624*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Mass ranges", "Subsubsection",
 CellChangeTimes->{{3.58548326035649*^9, 3.585483262384583*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "Set", " ", "Goldstone", " ", "mass", " ", "ranges", " ", "to", " ", "be", 
    " ", "zero"}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"i", "=", "5"}], ",", 
    RowBox[{"i", "<", "Nreal"}], ",", 
    RowBox[{"++", "i"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"massMin", "[", "i", "]"}], "=", "0"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"massMax", "[", "i", "]"}], "=", "0"}], ";"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.592292191015265*^9, 3.59229221112359*^9}, {
  3.592292263678061*^9, 3.59229226415022*^9}, {3.6011897267328253`*^9, 
  3.601189743197155*^9}}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Specific", " ", "mass", " ", "for", " ", "a", " ", "certain", " ", 
   "particle"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.585483301893355*^9, 3.585483313515202*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"massMin", "[", "0", "]"}], "=", "125"}], ";"}], " ", 
  RowBox[{"(*", 
   RowBox[{
   "Observed", " ", "Higgs", " ", "mass", " ", "set", " ", "to", " ", "125"}],
    "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"massMax", "[", "0", "]"}], "=", "125"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"massMin", "[", "1", "]"}], "=", "50"}], ";"}], " ", 
  RowBox[{"(*", 
   RowBox[{"Other", " ", "Higgs"}], "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"massMax", "[", "1", "]"}], "=", "125"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"massMin", "[", "2", "]"}], "=", "50"}], ";"}], " ", 
  RowBox[{"(*", "A", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"massMax", "[", "2", "]"}], "=", "500"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"massMin", "[", "3", "]"}], "=", "50"}], ";"}], " ", 
  RowBox[{"(*", 
   RowBox[{"Re", 
    RowBox[{"(", 
     RowBox[{"H", "+"}], ")"}]}], "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"massMax", "[", "3", "]"}], "=", "500"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"massMin", "[", "4", "]"}], "=", "50"}], ";"}], " ", 
  RowBox[{"(*", 
   RowBox[{"Im", 
    RowBox[{"(", 
     RowBox[{"H", "+"}], ")"}]}], "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"massMax", "[", "4", "]"}], "=", "500"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"massMin", "[", "5", "]"}], "=", "0"}], ";"}], " ", 
  RowBox[{"(*", 
   RowBox[{"Goldstone", " ", "1"}], "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"massMax", "[", "5", "]"}], "=", "0"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"massMin", "[", "6", "]"}], "=", "0"}], ";"}], " ", 
  RowBox[{"(*", 
   RowBox[{"Goldstone", " ", "2"}], "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"massMax", "[", "6", "]"}], "=", "0"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"massMin", "[", "7", "]"}], "=", "0"}], ";"}], " ", 
  RowBox[{"(*", 
   RowBox[{"Goldstone", " ", "3"}], "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"massMax", "[", "7", "]"}], "=", "0"}], ";"}]}], "Input",
 CellChangeTimes->{{3.549619343086762*^9, 3.5496193554912987`*^9}, {
   3.549621392217161*^9, 3.549621392246396*^9}, {3.57078233320813*^9, 
   3.570782333509603*^9}, {3.577100612135137*^9, 3.5771006133111057`*^9}, 
   3.583126822205221*^9, {3.587798764476585*^9, 3.587798765427034*^9}, {
   3.5922898185197887`*^9, 3.592289862180032*^9}, {3.592289899636304*^9, 
   3.592290078794586*^9}, {3.5922901357887173`*^9, 3.592290142961578*^9}, {
   3.5922901926812077`*^9, 3.592290196497408*^9}, {3.592291343690559*^9, 
   3.592291346986027*^9}, {3.592291492023884*^9, 3.5922915101992893`*^9}, {
   3.59229222874465*^9, 3.592292301362009*^9}, {3.592292771613163*^9, 
   3.592292807907906*^9}, {3.5923016131994457`*^9, 3.5923016268463593`*^9}, {
   3.592302672627886*^9, 3.592302682779312*^9}, {3.59230312074156*^9, 
   3.5923031240366364`*^9}, {3.592306495349523*^9, 3.592306497858595*^9}, {
   3.592317502934882*^9, 3.592317511601275*^9}, {3.592317543060528*^9, 
   3.592317547516808*^9}, {3.592317592096477*^9, 3.592317594466666*^9}, {
   3.592318432592949*^9, 3.592318434678055*^9}, {3.592318520893559*^9, 
   3.5923185246108103`*^9}, {3.592318580204406*^9, 3.59231859702777*^9}, {
   3.5923186330449047`*^9, 3.592318642325233*^9}, {3.592318785102261*^9, 
   3.592318794573059*^9}, {3.5923188793731318`*^9, 3.5923189112828207`*^9}, {
   3.592318958690618*^9, 3.592318960371489*^9}, {3.592319184406159*^9, 
   3.592319185927623*^9}, {3.5923193714884987`*^9, 3.592319377691205*^9}, {
   3.595334677533042*^9, 3.595334682109954*^9}, {3.5953361071721087`*^9, 
   3.595336133783901*^9}, {3.601101688586821*^9, 3.601101705447826*^9}, {
   3.601189726768148*^9, 3.6011897430665617`*^9}, {3.602919043544359*^9, 
   3.602919043851573*^9}, {3.602919083472353*^9, 3.602919128810039*^9}, 
   3.60291928314499*^9, {3.602920219178535*^9, 3.602920224569525*^9}, {
   3.6029202639579477`*^9, 3.602920293489231*^9}, {3.603543658496427*^9, 
   3.6035436642532663`*^9}, {3.603543743346271*^9, 3.60354377158597*^9}, {
   3.603543936214473*^9, 3.603543940152082*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Parametrized Mass/Coupling relations (optional --  ON if  NparamsML > 0)\
\>", "Subsubsection",
 CellChangeTimes->{{3.5854833856917877`*^9, 3.585483428222928*^9}, 
   3.6011288815427237`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "Initializing", " ", "all", " ", "ranges", " ", "to", " ", "zero"}], " ", 
   "*)"}], "\[IndentingNewLine]", 
  RowBox[{"For", "[", 
   RowBox[{
    RowBox[{"i", "=", "0"}], ",", 
    RowBox[{"i", "<", "NparamsML"}], " ", ",", 
    RowBox[{"++", "i"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"MLParMin", "[", "i", "]"}], "=", "0"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"MLParMax", "[", "i", "]"}], "=", "0"}], ";"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 CellChangeTimes->{{3.583126581349114*^9, 3.5831266062864437`*^9}, {
   3.5837585897426977`*^9, 3.5837586000189342`*^9}, {3.585483442537485*^9, 
   3.585483443350686*^9}, 3.6011288815710583`*^9}],

Cell[BoxData[
 RowBox[{"(*", " ", 
  RowBox[{
  "Set", " ", "range", " ", "for", " ", "a", " ", "specific", " ", 
   "parameter"}], " ", "*)"}]], "Input",
 CellChangeTimes->{{3.585483453144602*^9, 3.5854834838163023`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"MLParMin", "[", "0", "]"}], "=", "0"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"MLParMax", "[", "0", "]"}], "=", "400"}], ";"}]}], "Input",
 CellChangeTimes->{{3.549640419086753*^9, 3.549640426375087*^9}, {
  3.549640544747589*^9, 3.549640545463179*^9}, {3.5830718657475977`*^9, 
  3.583071880718238*^9}, {3.583071916677647*^9, 3.583071918359364*^9}, {
  3.583071955764485*^9, 3.5830719559831743`*^9}, {3.583072728597533*^9, 
  3.583072743941885*^9}, {3.58308406561486*^9, 3.5830840673143587`*^9}, {
  3.5831198193998013`*^9, 3.583119833680192*^9}, {3.583126620567246*^9, 
  3.5831266357285147`*^9}, {3.583758520617651*^9, 3.583758522844577*^9}, {
  3.5837586019895277`*^9, 3.583758604048403*^9}, {3.583758753965784*^9, 
  3.5837587626114902`*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Name of the file where to write", "Subsection",
 CellChangeTimes->{{3.534140116963636*^9, 3.534140127398847*^9}, {
   3.534140263482255*^9, 3.534140264947093*^9}, 3.601101018171115*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"InputFileName", "=", "\"\<model.in\>\""}], ";"}]], "Input",
 CellChangeTimes->{{3.58548137552264*^9, 3.585481390727833*^9}, {
  3.585657014150157*^9, 3.585657014295321*^9}, {3.585657801196809*^9, 
  3.585657801516176*^9}, {3.585657904770303*^9, 3.5856579048798237`*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Final call to generate input file (do not change this section!)\
\>", "Section",
 CellChangeTimes->{{3.534139828687394*^9, 3.5341398452615843`*^9}, {
  3.534144775282943*^9, 3.534144775694236*^9}, {3.534152310951947*^9, 
  3.5341523294879293`*^9}, {3.585479958037787*^9, 3.5854799904383173`*^9}, {
  3.585480023598304*^9, 3.58548002822314*^9}, {3.585480218772586*^9, 
  3.585480222969017*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"PathToScannerSDir", "=", 
   RowBox[{"NotebookDirectory", "[", "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.601128754920636*^9, 3.6011288032721443`*^9}}],

Cell[BoxData[
 RowBox[{"<<", 
  RowBox[{"(", 
   RowBox[{"StringJoin", "[", 
    RowBox[{
    "PathToScannerSDir", ",", "\"\<ScannerScore/ScannerSInputPackage.m\>\""}],
     "]"}], ")"}]}]], "Input",
 CellChangeTimes->{{3.585657443810758*^9, 3.585657482848139*^9}, {
   3.5856577377167387`*^9, 3.585657792996539*^9}, {3.585657844130536*^9, 
   3.585657845703484*^9}, 3.601099672234502*^9, {3.601099713043347*^9, 
   3.601099758678671*^9}, 3.601099822621681*^9, {3.601100958982444*^9, 
   3.601101000289703*^9}, 3.601128792174238*^9}]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1029, 691},
WindowMargins->{{Automatic, 0}, {Automatic, 0}},
Magnification:>FEPrivate`If[
  FEPrivate`Equal[FEPrivate`$VersionNumber, 6.], 1.25, 1.25 Inherited],
FrontEndVersion->"9.0 for Linux x86 (64-bit) (February 7, 2013)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 163, 2, 116, "Title"],
Cell[733, 26, 3390, 84, 628, "Text"],
Cell[CellGroupData[{
Cell[4148, 114, 185, 2, 145, "Section"],
Cell[CellGroupData[{
Cell[4358, 120, 234, 3, 56, "Subsection"],
Cell[4595, 125, 664, 14, 64, "Input"],
Cell[5262, 141, 895, 17, 64, "Input"],
Cell[6160, 160, 2222, 44, 189, "Input"],
Cell[8385, 206, 2323, 43, 239, "Input"],
Cell[10711, 251, 2620, 52, 314, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[13368, 308, 340, 6, 58, "Subsection"],
Cell[13711, 316, 248, 5, 63, "Text"],
Cell[CellGroupData[{
Cell[13984, 325, 1092, 25, 38, "Input"],
Cell[15079, 352, 4427, 71, 64, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[19543, 428, 1291, 28, 38, "Input"],
Cell[20837, 458, 5528, 86, 64, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[26402, 549, 852, 22, 38, "Input"],
Cell[27257, 573, 4388, 70, 64, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[31682, 648, 759, 20, 38, "Input"],
Cell[32444, 670, 4825, 76, 64, "Output"]
}, Open  ]],
Cell[37284, 749, 330, 6, 63, "Text"],
Cell[CellGroupData[{
Cell[37639, 759, 4485, 90, 114, "Input"],
Cell[42127, 851, 22161, 594, 1068, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[64337, 1451, 195, 2, 56, "Subsection"],
Cell[64535, 1455, 693, 19, 139, "Input"],
Cell[65231, 1476, 508, 11, 114, "Input"],
Cell[65742, 1489, 461, 11, 64, "Input"],
Cell[66206, 1502, 555, 12, 64, "Input"],
Cell[CellGroupData[{
Cell[66786, 1518, 162, 3, 44, "Subsubsection"],
Cell[66951, 1523, 139, 3, 38, "Input"],
Cell[67093, 1528, 511, 14, 114, "Input"],
Cell[67607, 1544, 507, 11, 89, "Input"],
Cell[68117, 1557, 1385, 24, 64, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[69539, 1586, 242, 4, 75, "Subsubsection"],
Cell[69784, 1592, 139, 3, 38, "Input"],
Cell[69926, 1597, 576, 15, 114, "Input"],
Cell[70505, 1614, 507, 11, 89, "Input"],
Cell[71015, 1627, 911, 17, 64, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[71975, 1650, 132, 1, 58, "Subsection"],
Cell[CellGroupData[{
Cell[72132, 1655, 106, 1, 44, "Subsubsection"],
Cell[72241, 1658, 3947, 89, 464, "Input"],
Cell[76191, 1749, 1165, 26, 189, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[77393, 1780, 101, 1, 44, "Subsubsection"],
Cell[77497, 1783, 758, 20, 139, "Input"],
Cell[78258, 1805, 222, 5, 38, "Input"],
Cell[78483, 1812, 4479, 112, 414, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[82999, 1929, 201, 4, 75, "Subsubsection"],
Cell[83203, 1935, 779, 20, 139, "Input"],
Cell[83985, 1957, 222, 5, 38, "Input"],
Cell[84210, 1964, 823, 16, 64, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[85082, 1986, 191, 2, 56, "Subsection"],
Cell[85276, 1990, 305, 5, 38, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[85630, 2001, 403, 7, 145, "Section"],
Cell[86036, 2010, 191, 4, 38, "Input"],
Cell[86230, 2016, 533, 11, 38, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
